import Orange.bio.gene.homology
import Orange.bio.taxonomy

data = Orange.data.Table("brown-selected")

geneinfo = Orange.bio.gene.NCBIGeneInfo('4932')

genes = [str(ex["gene"]) for ex in data]

for gene in genes:
    mappedgene = Orange.bio.gene.homology.homolog(geneinfo(gene).symbol, \
        '4932', '9606')
    print gene, mappedgene



"""
========================================
Gene Expression Atlas (``obiGeneAtlas``)
========================================

.. warning::

    Deprecated!!!
    Gene Expression Atlas REST api has been deprecated and will be removed
    in the future.


Interface to Gene Expression Atlas.

`Gene Expression Atlas <http://www.ebi.ac.uk/gxa/>`_ is a curated subset of
gene expression experiments in Array Express Archive.


.. autofunction:: gene_expression_atlas

.. autofunction:: default_gene_matcher

.. autofunction:: to_taxid

"""

from __future__ import absolute_import

import os
import shelve
import warnings
from collections import defaultdict, namedtuple
from contextlib import closing, contextmanager

from Orange.utils import serverfiles

from . import obiGene

GeneResults = namedtuple("GeneResults", "id name synonyms expressions")
ExpressionResults = namedtuple("ExpressionResults", "ef efv up down experiments")
ExperimentExpression = namedtuple("ExperimentExpression", "accession expression pvalue")

##
GeneAtlasResult = GeneResults
AtlasExpressions = ExpressionResults
AtlasExperiment = ExperimentExpression
##

CACHE_VERSION = 1


def _cache(name="AtlasGeneResult.shelve"):
    """ Return a open cache instance (a shelve object).
    """
    if not os.path.exists(serverfiles.localpath("GeneAtlas")):
        try:
            os.makedirs(serverfiles.localpath("GeneAtlas"))
        except OSError:
            pass
    cache = shelve.open(serverfiles.localpath("GeneAtlas", name))
    if cache.get(name + "__CACHE_VERSION__", None) == CACHE_VERSION:
        return cache
    else:
        cache.close()
        cache = shelve.open(serverfiles.localpath("GeneAtlas", name), "n")
        cache[name + "__CACHE_VERSION__"] = CACHE_VERSION
        return cache


SLEEP_TIME_MULTIPLIER = 3.0

def gene_expression_atlas(genes, progress_callback=None):
    """ Return GeneResults instances for genes (genes must be valid ensembl ids).
    """
    import time
    genes = list(genes)
    result_dict = {}
    genes_not_cached = []
    # See which genes are already cached
    with closing(_cache()) as cache:
        for gene in genes:
            if str(gene) in cache:
                result_dict[gene] = cache[str(gene)]
            else:
                genes_not_cached.append(gene)
    
    batch_size = 10
    start = 0
    res = []
    while start < len(genes_not_cached):
        batch = genes_not_cached[start: start + batch_size]
        start += batch_size
        start_time = time.time()
        batch_res = batch_gene_atlas_expression(batch)
        # Cache the new results.
        # TODO: handle genes without any results.
        genes_with_no_results = set(batch) - set(r.id for r in batch_res) 
        with closing(_cache()) as cache:
            for atlas_res in batch_res:
                cache[str(atlas_res.id)] = atlas_res
                result_dict[atlas_res.id] = atlas_res
            for g in genes_with_no_results:
                cache[str(g)] = None
        res.extend(batch_res)
        # Sleep
        if start % (batch_size * 10) == 0:
            # every 10 batches wait one minute before continuing. 
            time.sleep(60)
        else:
            time.sleep(min(20.0, SLEEP_TIME_MULTIPLIER*(time.time() - start_time)))
            
        if progress_callback:
            progress_callback(100.0 * start / len(genes_not_cached))
    
    return [result_dict.get(g, None) for g in genes]

    
def batch_gene_atlas_expression(genes):
    cond = GenePropertyCondition("Ensgene", "Is", genes)
    res = run_query(cond, format="json")
    results = res["results"]
    results_genes = []
    for one_result in results:
        gene = one_result["gene"]
        id = gene["id"]
        name = gene["name"]
        synonyms = gene.get("synonyms", [])
        expressions = one_result["expressions"]
        result_expressions = []
        for expression in expressions:
            ef = expression["ef"]
            efv = expression["efv"]
            up = expression["upExperiments"]
            down = expression["downExperiments"]
            experiments = expression["experiments"]
            result_experiment = []
            for exp in experiments:
                if "accession" in exp:
                    exp_accession = exp["accession"]
                elif "experimentAccession" in exp:
                    exp_accession = exp["experimentAccession"]
                else:
                    raise KeyError()
                if "expression" in exp:
                    updown = exp["expression"]
                elif "updn" in exp:
                    updown = exp["updn"]
                else:
                    raise KeyError
                pval = exp["pvalue"]
                result_experiment.append(ExperimentExpression(exp_accession, updown, pval))
            result_expressions.append(ExpressionResults(ef, efv, up, down, result_experiment))
        results_genes.append(GeneResults(id, name, synonyms, result_expressions))
    return results_genes


def default_gene_matcher(organism):
    """ Return a default gene matcher for organism
    (targeting Ensembl gene ids).
    
    """
    taxid = to_taxid(organism)
    matcher = obiGene.matcher(
      [obiGene.GMEnsembl(taxid),
       [obiGene.GMEnsembl(taxid),
        obiGene.GMNCBI(taxid)]]
    )
    matcher.set_targets(obiGene.EnsembleGeneInfo(taxid).keys())
    return matcher

from Orange.utils import lru_cache


@lru_cache(maxsize=3)
def _cached_default_gene_matcher(organism):
    return default_gene_matcher(organism)
    

def get_atlas_summary(genes, organism, gene_matcher=None,
                      progress_callback=None):
    """ Return 3 dictionaries containing a summary of atlas information
    about three experimental factors:
    
        - Organism Part (OP) 
        - Disease State (DS)
        - Cell type (CT)
    
    Each dictionary contains query genes as keys. Values are dictionaries
    mapping factor values to a 2-tuple containig the count of up regulated
    and down regulated experiments.
    
    Example ::
    
        >>> get_atlas_summary(["ENSG00000159216"], "Homo sapiens")
        ({u'RUNX1': ...
        
    """
    if gene_matcher is None:
        gene_matcher = _cached_default_gene_matcher(organism)
        
    matched, unmatched = [], []
    for gene, match in zip(genes, map(gene_matcher.umatch, genes)):
        if match:
            matched.append(match)
        else:
            unmatched.append(gene)
    if unmatched:
        warnings.warn("Unmatched genes " + "," .join(["%r" % g for g in unmatched]))
    
    results = gene_expression_atlas(matched, progress_callback=progress_callback)
    
    def collect_ef_summary(result, ef, summary):
        for exp in result.expressions:
            if exp.ef == ef:
                if any([exp.up, exp.down]):
                    summary[result.name][exp.efv] = (exp.up, exp.down)
        
            
    op, ds, ct = defaultdict(dict), defaultdict(dict), defaultdict(dict)
    for res in results:
        if res:
            collect_ef_summary(res, "organism_part", op)
            collect_ef_summary(res, "disease_state", ds)
            collect_ef_summary(res, "cell_type", ct)
        
    return dict(op), dict(ds), dict(ct)

def drop_none(iter):
    """ Drop all ``None`` from the iterator. 
    """
    for e in iter:
        if e is not None:
            yield e
            
def construct_atlas_gene_sets(genes, organism, factors=["organism_part",
                                    "disease_state", "cell_type"],
                              max_pvalue=1e-5):
    """ Construct gene sets for atlas experimental factor values in
    ``factors``.
    """
    results = gene_expression_atlas(genes)
    sets = defaultdict(list)
    
    for res in drop_none(results):
        for exp in res.expressions:
            if exp.ef not in factors:
                continue
            diff_exp = [e for e in exp.experiments \
                        if e.pvalue <= max_pvalue]
            if diff_exp:
                sets[exp.ef, exp.efv].append(res.id)

    organism = "+".join(organism.lower().split())
    from .obiGeneSets import GeneSets, GeneSet
    
    def display_string(name):
        return name.capitalize().replace("_", " ")
    
    gene_sets = []
    for (ef, efv), genes in sets.items():
        ef_display = display_string(ef)
        gs = GeneSet(genes, "Diff. expressed in %s=%s." % (ef_display, efv), id=ef + ":" + efv,
                     description="Diff. expressed in %s=%s" % (ef_display, efv),
                     link="http://www.ebi.ac.uk/gxa/qrs?specie_0={organism}&gprop_0=&gnot_0=&gval_0=&fact_1=&fexp_1=UPDOWN&fmex_1=&fval_1=%22{efv}%22+&view=hm".format( \
                            organism=organism, efv="+".join(efv.lower().split())),
                     hierarchy=("Gene expression atlas", ef_display))
        gene_sets.append(gs)
    return GeneSets(gene_sets)


# Mapping for common taxids from obiTaxonomy
TAXID_TO_ORG = {"": "Anopheles gambiae",
                "3702": "Arabidopsis thaliana",
                "9913": "Bos taurus",
                "6239": "Caenorhabditis elegans",
                "7955": "Danio rerio",
                "7227": "Drosophila melanogaster",
                "": "Epstein barr virus",
                "": "Gallus gallus",
                "9606": "Homo sapiens",
                "": "Human cytomegalovirus",
                "": "Kaposi sarcoma-associated herpesvirus",
                "10090": "Mus musculus",
                "10116": "Rattus norvegicus",
                "4932": "Saccharomyces cerevisiae",
                "4896": "Schizosaccharomyces pombe",
                "8355": "Xenopus laevis"
     }

def to_taxid(name):
    dd = dict((v, k) for k, v in TAXID_TO_ORG.items())
    if name in dd:
        return dd[name]
    else:
        from . import obiTaxonomy as tax
        ids = tax.to_taxid(name, mapTo=TAXID_TO_ORG.keys())
        if ids:
            return ids.pop()
        else:
            raise ValueError("Unknown organism.")


__doc__ += """\
Low level REST query interface
------------------------------

Use `query_atlas_simple` for simple querys.

Example (query human genes for experiments in which they are up regulated) ::

    >>> run_simple_query(genes=["SORL1", "PSIP1", "CDKN1C"], regulation="up", organism="Homo sapiens")
    {u'...
    
Or use the `AtlasCondition` subclasses in this module to construct a more
advanced query and use the `run_query` function.

Example (query human genes annotated to the GO term 'transporter activity'
that are up regulated in the liver in at least three experiments) ::

    >>> go_cond = GenePropertyCondition("Goterm", "Is", "transporter activity")
    >>> liver_cond = ExperimentalFactorCondition("Organism_part", "up", 3, "liver")
    >>> org_cond = OrganismCondition("Homo sapiens")
    >>> cond_list = ConditionList([go_cond, liver_cond, org_cond])
    >>> run_query(cond_list)
    {u'...

"""

import urllib2
from  StringIO import StringIO
import json
from xml.etree.ElementTree import ElementTree

parse_json = json.load


def parse_xml(stream):
    """ Parse an xml stream into an instance of xml.etree.ElementTree.ElementTree.
    """
    return ElementTree(file=stream)


class GeneExpressionAtlasConenction(object):
    """
    A connection to Gene Expression Atlas database.

    :param address:
        Address of the GXA server (default: http://www-test.ebi.ac.uk/gxa/api/deprecated).
    :param timeout:
        Socket timeout (default 30).
    :param cache:
        A dict like object to use as a cache.

    """
    DEFAULT_ADDRESS = "http://www-test.ebi.ac.uk/gxa/api/deprecated"
    DEFAULT_CACHE = serverfiles.localpath(
        "GeneAtlas", "GeneAtlasConnectionCache.shelve")

    def __init__(self, address=None, timeout=30, cache=None):

        self.address = address if address is not None else self.DEFAULT_ADDRESS
        self.timeout = timeout
        self.cache = cache if cache is not None else self.DEFAULT_CACHE

    def query(self, condition, format="json", start=None, rows=None, indent=False):
        warnings.warn(
            "The Gene Expression Atlas REST api has been deprecated and " +
            "will be removed in the future.",
            UserWarning)

        url = self.address + "?" + condition.rest()
        if start is not None and rows is not None:
            url += "&start={0}&rows={1}".format(start, rows)
        url += "&format={0}".format(format)
        if indent:
            url += "&indent"
        #print url

        if self.cache is not None:
            return self._query_cached(url, format)
        else:
            return urllib2.urlopen(url)

    def _query_cached(self, url, format):
        if self.cache is not None:
            with self.open_cache("r") as cache:
                if url in cache:
                    return StringIO(cache[url])

            response = urllib2.urlopen(url)
            contents = response.read()
            # Test if the contents is a valid json or xml string (some
            # times the stream just stops in the middle, so this makes
            # sure we don't cache an invalid response
            # TODO: what about errors (e.g. 'cannot handle the
            # query in a timely fashion'
            if format == "json":
                parse_json(StringIO(contents))
            else:
                parse_xml(StringIO(contents))

            with self.open_cache("w") as cache:
                cache[url] = contents

            return StringIO(contents)
        else:
            return urllib2.urlopen(url)

    def open_cache(self, flag="r"):
        """
        Return a context manager for a dict like object.
        """
        if isinstance(self.cache, basestring):
            try:
                return closing(_open_shelve(self.cache, flag))
            except Exception:
                return fake_closing({})
        else:
            return fake_closing(self.cache)


def _open_shelve(filename, flag="r"):
    dirname = os.path.dirname(filename)
    if not os.path.isdir(dirname):
        os.makedirs(dirname)
    exists = os.path.exists(filename)
    if flag in ["r", "w"] and not exists:
        # needs to be created first
        # XXX: Race condition
        s = shelve.open(filename, "c")
        s.close()

    return shelve.open(filename, flag)


@contextmanager
def fake_closing(obj):
    yield obj
    
    
# Names of all Gene Property filter names
GENE_FILTERS = \
    ["Name", # Gene name
     "Goterm", #Gene Ontology Term
     "Interproterm", #InterPro Term
     "Disease", #Gene-Disease Assocation
     "Keyword", #Gene Keyword
     "Protein", #Protein

     "Dbxref", #Other Database Cross-Refs
     "Embl", #EMBL-Bank ID
     "Ensfamily", #Ensembl Family
     "Ensgene", #Ensembl Gene ID

     "Ensprotein", #Ensembl Protein ID
     "Enstranscript", #Ensembl Transcript ID
     "Goid", #Gene Ontology ID
     "Image", #IMAGE ID
     "Interproid", #InterPro ID
     "Locuslink", #Entrez Gene ID

     "Omimid", #OMIM ID
     "Orf", #ORF
     "Refseq", #RefSeq ID
     "Unigene", #UniGene ID
     "Uniprot", #UniProt Accession

     "Hmdb", #HMDB ID
     "Chebi", #ChEBI ID
     "Cas", #CAS
     "Uniprotmetenz", #Uniprotmetenz
     "Gene", #Gene Name or Identifier
     "Synonym", #Gene Synonym
     ]
    
# Valid Gene Property filter qualifiers 
GENE_FILTER_QUALIFIERS =\
    ["Is",
     "IsNot"
     ]

# Organisms in the Atlas
ATLAS_ORGANISMS = \
    ["Anopheles gambiae",
     "Arabidopsis thaliana",
     "Bacillus subtilis",
     "Bos taurus",
     "Caenorhabditis elegans",
     "Canis familiaris",
     "Ciona intestinalis",
     "Ciona savignyi",
     "Danio rerio",
     "Dasypus novemcinctus",
     "Drosophila melanogaster",
     "Epstein barr virus",
     "Equus caballus",
     "Gallus gallus",
     "Gorilla gorilla",
     "Homo sapiens",
     "Human cytomegalovirus",
     "Human immunodeficiency virus",
     "Kaposi sarcoma-associated herpesvirus",
     "Macaca mulatta",
     "Monodelphis domestica",
     "Mus musculus",
     "Ornithorhynchus anatinus",
     "Oryza sativa",
     "Pan troglodytes",
     "Pongo abelii",
     "Populus trichocarpa",
     "Rattus norvegicus",
     "Saccharomyces cerevisiae",
     "Schizosaccharomyces pombe",
     "Sus scrofa",
     "Unknown",
     "Vitis vinifera",
     "Xenopus laevis",
     "Xenopus tropicalis"
     ]
    
def ef_ontology():
    """ Return the `EF <http://www.ebi.ac.uk/efo/>`_ (Experimental Factor) ontology
    """
    from . import obiOntology
    from Orange.utils import serverfiles
    # Should this be in the OBOFoundry (Ontology) domain
    try:
        file = open(serverfiles.localpath_download("ArrayExpress", "efo.obo"), "rb")
    except urllib2.HTTPError:
        file = urllib2.urlopen("http://efo.svn.sourceforge.net/svnroot/efo/trunk/src/efoinobo/efo.obo")
    return obiOntology.OBOOntology(file)


class Condition(object):
    """ Base class for Gene Expression Atlas query condition
    """
    def validate(self):
        """ Validate condition in a subclass.
        """
        raise NotImplementedError
    
    def rest(self):
        """ Return a REST query part in a subclass.
        """
        raise NotImplementedError
    
    
class ConditionList(list, Condition):
    """ A list of AtlasCondition instances.
    """ 
    def validate(self):
        for item in self:
            item.validate()
        
    def rest(self):
        return "&".join(cond.rest() for cond in self)


class GenePropertyCondition(Condition):
    """ An atlas gene filter condition.
    
    :param property: Property of the gene. If None or "" all properties 
        will be searched.
    :param qualifier: Qualifier can be 'Is' or 'IsNot'
    :param value: The value to search for.
    
    Example ::
    
        >>> # Condition on a gene name
        >>> condition = GenePropertyCondition("Name", "Is", "AS3MT")
        >>> # Condition on genes from a GO Term
        >>> condition = GenePropertyCondition("Goterm", "Is", "p53 binding")
        >>> # Condition on disease association
        >>> condition = GenePropertyCondition("Disease", "Is", "cancer")
        
    """
    def __init__(self, property, qualifier, value):
        self.property = property or ""
        self.qualifier = qualifier
        if isinstance(value, basestring):
            self.value = value.replace(" ", "+")
        elif isinstance(value, list):
            self.value = "+".join(value)
        else:
            raise ValueError(value)
        
        self.validate()
        
    def validate(self):
        assert(self.property in GENE_FILTERS + [""])
        assert(self.qualifier in GENE_FILTER_QUALIFIERS + [""])
        
    def rest(self):
        return "gene{property}{qualifier}={value}".format(**self.__dict__)
        
        
class ExperimentalFactorCondition(Condition):
    """ An atlas experimental factor filter condition.
    
    :param factor: EFO experiamntal factor
    :param regulation: "up", "down", "updown", "any" or "none"
    :param n: Minimum number of of experimants with this condition
    :param value: Experimantal factor value
    
    Example ::
    
        >>> # Any genes up regulated in at least 3 experiments involving cancer.
        >>> condition = ExperimentalFactorCondition("", "up", 3, "cancer")
        >>> # Only genes which are up/down regulated in the heart in at least one experiment. 
        >>> condition = ExperimentalFactorCondition("Organism_part", "updown", 1, "heart")
        
    """
    def __init__(self, factor, regulation, n, value):
        self.factor = factor
        self.regulation = regulation
        self.n = n
        self.value = value
        self.validate()
        
    def validate(self):
        # TODO: validate the factor and value
#        assert(self.factor in ef_ontology())
        assert(self.regulation in ["up", "down", "updown", "none", "any"])
        
    def rest(self):
        return "{regulation}{n}In{factor}={value}".format(**self.__dict__)
        
        
class OrganismCondition(Condition):
    """ Condition on organism.
    """
    def __init__(self, organism):
        self.organism = organism
        self.validate()
        
    def validate(self):
        assert(self.organism in ATLAS_ORGANISMS)
        
    def rest(self):
        return "species={0}".format(self.organism.replace(" ", "+").lower())
        
        
class ExperimentCondition(Condition):
    """ Condition on experiement
    
    :param property: Property of the experiment. If None or "" all properties 
        will be searched.
    :param qualifier: Qualifier can be 'Has' or 'HasNot'
    :param value: The value to search for.
    
    Example ::
    
        >>> # Condition on a experiemnt acession
        >>> condition = ExperimentCondition("", "", "E-GEOD-24283")
        >>> # Condition on experiments involving lung
        >>> condition = ExperimentCondition("Organism_part", "Has", "lung")
        
    """
    EXPERIMENT_FILTER_QUALIFIERS = [
                "Has",
                "HasNot"]
    
    def __init__(self, property, qualifier, value):
        self.property = property
        self.qualifier = qualifier
        if isinstance(value, basestring):
            self.value = value.replace(" ", "+")
        elif isinstance(value, list):
            self.value = "+".join(value)
        else:
            raise ValueError(value)
        
        self.validate()
        
    def validate(self):
        # TODO: check to EFO factors
#        assert(self.property in EXPERIMENT_FILTERS + [""])
        assert(self.qualifier in self.EXPERIMENT_FILTER_QUALIFIERS + [""])
        
    def rest(self):
        return "experiment{property}{qualifier}={value}".format(**self.__dict__)
        
        
class GeneExpressionAtlasError(Exception):
    """ An error response from the Atlas server.
    """
    pass
    
    
def __check_atlas_error_json(response):
    if "error" in response:
        raise GeneExpressionAtlasError(response["error"])
    return response
 
     
def __check_atlas_error_xml(response):
    error = response.find("error")
    if error is not None:
        raise GeneExpressionAtlasError(error.text)
    return response
    
        
def run_simple_query(genes=None, regulation=None, organism=None,
                     condition=None, format="json", start=None,
                     rows=None):
    """ A simple Gene Atlas query.
    
    :param genes: A list of gene names to search for.
    :param regulation: Search for experiments in which `genes` are "up",
        "down", "updown", "any" or "none" regulated. If "any" all experiments
        are searched.
    :param organism: Search experiments for organism. If None all experiments
        are searched.
    :param condition: An EFO factor value (e.g. "brain")
    
    Example ::
        
        >>> run_simple_query(genes=['Pou5f1', 'Dppa3'], organism="Mus musculus")
        {u'...
        
        >>> run_simple_query(genes=['Pou5f1', 'Dppa3'], regulation="up", organism="Mus musculus")
        {u'...
        
        >>> run_simple_query(genes=['Pou5f1', 'Dppa3'], regulation="up", condition="liver", organism="Mus musculus")
        {u'...
        
    """
    conditions = ConditionList()
    if genes:
        conditions.append(GenePropertyCondition("Gene", "Is", genes))
    if regulation or condition:
        regulation = "any" if regulation is None else regulation
        condition = "" if condition is None else condition
        conditions.append(ExperimentalFactorCondition("", regulation, 1, condition))
    if organism:
        conditions.append(OrganismCondition(organism))
        
    connection = GeneExpressionAtlasConenction()
    results = connection.query(conditions, format=format, start=start,
                               rows=rows)
    if format == "json":
        return parse_json(results)
    else:
        return parse_xml(results)

"""\
.. todo:: can this be implemented query_atlas(organism="...", Locuslink="...", Chebi="...", up3InCompound="..." downInEFO="...")
      Need a full list of accepted factors 
"""

def run_query(condition, format="json", start=None, rows=None, indent=False, connection=None):
    """ Query Atlas based on a `condition` (instance of :class:`Condition`)
    
    Example ::
        
        >>> condition1 = GenePropertyCondition("Goterm", "Is", "p53 binding")
        >>> condition2 = ExperimentalFactorCondition("Organism_part", "up", 3, "heart")
        >>> condition = ConditionList([condition1, condition2])
        >>> run_query(condition)
        {u'...
        
    """
    if connection is None:
        connection = GeneExpressionAtlasConenction()
    results = connection.query(condition, format=format, start=start,
                               rows=rows, indent=indent)
    if format == "json":
        response = parse_json(results)
        return __check_atlas_error_json(response)
    else:
        response = parse_xml(results)
        return __check_atlas_error_xml(response)
    
def test():
    from pprint import pprint    
    pprint(get_atlas_summary(['Pou5f1', 'Dppa3'], 'Mus musculus'))
       
    pprint(get_atlas_summary(['PDLIM5', 'FGFR2' ], 'Homo sapiens'))
    import doctest 
    doctest.testmod(optionflags=doctest.ELLIPSIS)
    
if __name__ == "__main__":
    test()



"""
<name>MeSH Browser</name>
<description>Browse MeSH ontology.</description>
<icon>icons/MeSHBrowser.svg</icon>
<contact>Crt Gorup (crt.gorup@gmail.com)</contact> 
<priority>2040</priority>
"""

from __future__ import absolute_import

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWWidget import *

from ..obiMeSH import *

NAME = "MeSH Browser"
DESCRIPTION = "Browse MeSH ontology."
ICON = "icons/MeSHBrowser.svg"
PRIORITY = 2040

INPUTS = [("Reference data", Orange.data.Table, "getReferenceData"),
          ("Cluster data", Orange.data.Table, "getClusterData")]

OUTPUTS = [("Selected examples", Orange.data.Table)]

REPLACES = ["_bioinformatics.widgets.OWMeSHBrowser.OWMeSHBrowser"]


class MyQTableWidgetItem(QTableWidgetItem):

    """ Our implementation of QTable item allowing numerical sorting.  """

    def __init__(self, table, text):
        QTableWidgetItem.__init__(self, table, QTableWidgetItem.Never, text)
        self.data = text

    def key(self):	  # additional setting to correctly handle text and numerical sorting
        try:	# sorting numerical column
            e = float(self.data)
            tdata = pval = "%.4g" % e
            l = len(self.data)
            pre = ""

            offset = 0	# additional parameter to hadle exponent '2e-14' float format
            if(tdata.count('e') > 0 or tdata.count('E') > 0):
                pre = "*"
                offset = 1

            for i in range(0, 40 - l - offset):
                pre = pre + "0"
            return pre + tdata
        except ValueError:	  # sorting text column
            return self.data


class ListViewToolTip(QToolTip):

    """brief A class to allow tooltips in a listview."""

    def __init__(self, view, column, data):
        """brief ListViewToolTip constructor.
        \param view		  QListView instance
        \param column		Listview column
        \param truncatedOnly Only display the tooltip if the column data is truncated
        """
        QToolTip.__init__(self, view.viewport())
        self.__view = view
        self.__col = column
        self.__data = data	  # mapping from name -> description
        # self.setWakeUpDelay(400)

    def appendData(self, data):
        self.__data = data

    def maybeTip(self, pos):
        """brief Draw the tooltip.
        \param pos Tooltip position.
        """
        item = self.__view.itemAt(pos)
        if item is not None:
            if(self.__data.has_key(str(item.text(self.__col)))):
                tipString = self.__data[str(item.text(self.__col))]
                counter = 45
                newTipString = ""
                for i in tipString.split(" "):
                    if counter < 0:
                        newTipString = newTipString + i + "\n"
                        counter = 45
                    else:
                        newTipString = newTipString + i + " "
                        counter -= len(i)
                tipString = newTipString
                cr = self.__view.itemRect(item)
                headerPos = self.__view.header().sectionPos(self.__col)
                cr.setLeft(headerPos)
                cr.setRight(headerPos + self.__view.header().sectionSize(self.__col))
                self.tip(cr, tipString)
            else:
                print item.text(self.__col) + " not found in toDecs"


class OWMeSHBrowser(OWWidget):
    settingsList = ["multi", "maxPValue", "minExamplesInTerm"]

    def __init__(self, parent=None, signalManager=None):
        OWWidget.__init__(self, parent, signalManager, "MeshBrowser")
        self.inputs = [("Reference data", ExampleTable, self.getReferenceData), ("Cluster data", ExampleTable, self.getClusterData)]
        self.outputs = [("Selected examples", ExampleTable)]

        # widget variables
        self.loadedRef = 0
        self.loadedClu = 0
        self.maxPValue = 0.05
        self.minExamplesInTerm = 5
        self.multi = 1
        self.reference = None
        self.cluster = None
        self.loadSettings()
        self.mesh = obiMeSH() # main object is created
        self.dataLoaded = self.mesh.dataLoaded

        # left pane
        box = OWGUI.widgetBox(self.controlArea, "Info")
        #box = QGroupBox("Info", self.controlArea)
        self.infoa = OWGUI.label(box, self, "No reference data.")
        self.infob = OWGUI.label(box, self, "No cluster data.")
        self.ratio = OWGUI.label(box, self, "")
        self.ref_att = OWGUI.label(box, self, "")
        self.clu_att = OWGUI.label(box, self, "")
        self.resize(960, 600)
        OWGUI.separator(self.controlArea)

        self.optionsBox = OWGUI.widgetBox(self.controlArea, "Options")
        self.maxp = OWGUI.lineEdit(self.optionsBox, self, "maxPValue", label="threshold:", orientation="horizontal", labelWidth=120, valueType=float)
        self.minf = OWGUI.lineEdit(self.optionsBox, self, "minExamplesInTerm", label="min. frequency:", orientation="horizontal", labelWidth=120, valueType=int)
        #OWGUI.checkBox(self.optionsBox, self, 'multi', 'Multiple selection', callback= self.checkClicked)
        OWGUI.button(self.optionsBox, self, "Refresh", callback=self.refresh)

        # right pane
        self.col_size = [280, 84, 84, 100, 110]
        self.sort_col = 0
        self.sort_dir = True
        self.columns = ['MeSH term', '# reference', '# cluster', 'p value', 'fold enrichment'] # both datasets

        self.splitter = QSplitter(Qt.Vertical, self.mainArea)
        self.mainArea.layout().addWidget(self.splitter)

        # list view
        self.meshLV = QTreeWidget(self.splitter)
        #self.meshLV.setSelectionMode(QAbstractItemView.MultiSelection)
        self.meshLV.setAllColumnsShowFocus(1)
        self.meshLV.setColumnCount(len(self.columns))
        self.meshLV.setHeaderLabels(self.columns)

        self.meshLV.header().setClickable(True)
        #self.meshLV.header().setSortIndicatorShown(True)
        #self.meshLV.setSortingEnabled(True)
        self.meshLV.setRootIsDecorated(True)
        self.connect(self.meshLV, SIGNAL("itemSelectionChanged()"), self.viewSelectionChanged)
        #self.meshLV.setItemDelegateForColumn(3, EnrichmentColumnItemDelegate(self))
        #self.tooltips = ListViewToolTip(self.meshLV,0, self.mesh.toDesc)

        # table of significant mesh terms
        self.sigTermsTable = QTableWidget(self.splitter)
        self.sigTermsTable.setColumnCount(len(self.columns))
        self.sigTermsTable.setRowCount(4)
        ## hide the vertical header
        self.sigTermsTable.verticalHeader().hide()
        #self.sigTermsTable.setLeftMargin(0)
        #self.sigTermsTable.setSelectionMode(QAbstractItemView.MultiSelection)

        for i in range(0, len(self.columns)):
            self.sigTermsTable.horizontalHeader().resizeSection(i, self.col_size[i])
            self.meshLV.header().resizeSection(i, self.col_size[i])

        self.sigTermsTable.setHorizontalHeaderLabels(self.columns)

        self.connect(self.sigTermsTable, SIGNAL("itemSelectionChanged()"), self.tableSelectionChanged)
        self.connect(self.sigTermsTable, SIGNAL("clicked(int,int,int,const QPoint&)"), self.tableClicked)
        self.splitter.show()
        self.optionsBox.setDisabled(1)

    def tableSelectionChanged(self):
        return True

    def tableClicked(self, row, col, button, point):
        if self.sort_col == col:
            self.sort_dir = not self.sort_dir
        else:
            self.sort_col = col
            self.sort_dir = True

        self.sigTermsTable.sortItems(self.sort_col, Qt.DescendingOrder)
        #print "sortiram ", col, " ",row

    def checkClicked(self):
        if self.multi == 0:
            self.meshLV.clearSelection()
        self.meshLV.setSelectionMode(QAbstractItemView.MultiSelection)

    def viewSelectionChanged(self):
        """
        Function viewSelectionChanged is used to handle the widget output once the user clicks or selects MeSH term inside the tree view.
        """
        items = list()
        self.progressBarInit()

        itms = self.meshLV.selectedItems()
        for i in itms:
            items.append(i.term)

        #for i in self.lvItem2Mesh.iterkeys():
        #	if i.isSelected():
        #		items.append(self.lvItem2Mesh[i][1])
        #print "selecting ", items

        if self.reference:
            data = self.mesh.findSubset(self.reference, items, callback=self.progressBarSet, MeSHtype='term')
        else:
            data = self.mesh.findSubset(self.cluster, items, callback=self.progressBarSet, MeSHtype='term')

        #print items
        self.send("Selected examples", data)
        self.progressBarFinished()

    def __updateData__(self):
        """
        Function __updateData__ is used to display the results of the MeSH term enrichment analysis inside the widget components.
        """
        self.lvItem2Mesh = dict()

        if(self.reference and self.cluster):
            if(len(self.cluster) > len(self.reference)):
                self.optionsBox.setDisabled(1)
                return False
            # everything is ok, now we can calculate enrichment and update labels, tree view and table data
            #self.warning()
            self.optionsBox.setDisabled(0)
            self.progressBarInit()
            self.treeInfo, self.results = self.mesh.findEnrichedTerms(self.reference, self.cluster, self.maxPValue, treeData= True, callback= self.progressBarSet)
            self.progressBarFinished()
            self.ratio.setText("ratio = %.4g" % self.mesh.ratio)
            self.clu_att.setText("cluster MeSH att: " + self.mesh.clu_att)
            self.ref_att.setText("reference MeSH att: " + self.mesh.ref_att)

            # table data update
            self.sigTermsTable.setRowCount(len(self.results))
            index = 0
            for i in self.results.iterkeys(): ## sorted by the p value
               # mTerm = i[0]
                mID = self.mesh.toName[i] + " (" + i + ")"
                rF = self.results[i][0]
                cF = self.results[i][1]
                pval = self.results[i][2]
                fold = self.results[i][3]
                pval = "%.4g" % pval
                fold = "%.4g" % fold
                vals = [mID, rF, cF, pval, fold]
                for j in range(len(vals)):
                    self.sigTermsTable.setItem(index, j, QTableWidgetItem(str(vals[j])))
                index = index + 1

            # initial sorting - p value
            self.sort_col = 3
            self.sort_dir = True
            self.sigTermsTable.sortItems(self.sort_col, Qt.DescendingOrder)

            # tree view update - The most beautiful part of this widget!
            starters = self.treeInfo["tops"]	   # we get a list of possible top nodes
            self.meshLV.clear()

            for e in starters:	  # we manualy create top nodes
                f = QTreeWidgetItem(self.meshLV);
                #f.setOpen(1)
                rfr = str(self.results[e][0])
                cfr = str(self.results[e][1])
                pval = "%.4g" % self.results[e][2]
                fold = "%.4g" % self.results[e][3]
                self.lvItem2Mesh[f] = (self.mesh.toName[e], e)
                data = [self.mesh.toName[e], rfr, cfr, pval, fold]
                f.term = self.mesh.toName[e]
                for t in range(len(data)):
                    f.setText(t, data[t])
                self.__treeViewMaker__(f, e, False)
            self.meshLV.expandAll()

        elif self.reference or self.cluster:
            if self.reference:
                current_data = self.reference
            else:
                current_data = self.cluster

            self.optionsBox.setDisabled(0)
            self.progressBarInit()
            self.treeInfo, self.results = self.mesh.findFrequentTerms(current_data, self.minExamplesInTerm, treeData=True, callback=self.progressBarSet)
            self.progressBarFinished()
            if self.reference:
                self.ref_att.setText("reference MeSH att: " + self.mesh.solo_att)
            else:
                self.clu_att.setText("cluster MeSH att: " + self.mesh.solo_att)

            # table data update
            self.sigTermsTable.setRowCount(len(self.results))
            index = 0
            for i in self.results.iterkeys():
                mID = self.mesh.toName[i] + " (" + i + ")"
                rF = self.results[i]
                vals = [mID, rF]
                for j in range(len(vals)):
                    self.sigTermsTable.setItem(index, j, QTableWidgetItem(str(vals[j])))
                index = index + 1

            # initial sorting - frequency
            self.sort_col = 1
            self.sort_dir = True
            self.sigTermsTable.sortItems(self.sort_col, Qt.DescendingOrder)

            # tree view update - The most beautiful part of this widget!
            starters = self.treeInfo["tops"]	   # we get a list of possible top nodes
            self.meshLV.clear()

            for e in starters:	  # we manualy create top nodes
                f = QTreeWidgetItem(self.meshLV);
                #f.setOpen(1)
                self.lvItem2Mesh[f] = (self.mesh.toName[e], e)
                rfr = str(self.results[e])
                data = [self.mesh.toName[e], rfr]
                f.term = self.mesh.toName[e]
                for t in range(len(data)):
                    f.setText(t, data[t])
                self.__treeViewMaker__(f, e, True)
            self.meshLV.expandAll()

    def __treeViewMaker__(self, parentLVI, parentID, soloMode):
        """
        Function __treeViewMaker__ is used to build the tree in treeListView. When soloMode=True function only displays \
        first two columns inside the tree view (suitable when only one dataset is present).
        """
        for i in self.treeInfo[parentID]:   # for each succesor
            f = QTreeWidgetItem(parentLVI);
            f.term = self.mesh.toName[i]
            data = [self.mesh.toName[i]]
            if soloMode:
                rfr = str(self.results[i])
                data.append(rfr)
            else:		   # when we have referece and cluster dataset we have to print additional info
                rfr = str(self.results[i][0])
                cfr = str(self.results[i][1])
                pval = "%.4g" % self.results[i][2]
                fold = "%.4g" % self.results[i][3]
                data.extend([rfr, cfr, pval, fold])
            self.lvItem2Mesh[f] = (data[0], i)		# mapping   QTreeWidgetItem <-> mesh id
            for t in range(len(data)):
                f.setText(t, data[t])
            self.__treeViewMaker__(f, i, soloMode)
        return True

    def refresh(self):
        """
        Function refresh is executed as number of widget inputs changes.
        """
        if self.reference or self.cluster:
            self.__switchGUI__()
            self.__updateData__()

    def __clearGUI__(self):
        """
        Function __clearGUI__ sets tree view, table and labels to their default values.
        """
        self.meshLV.clear()
        self.sigTermsTable.setRowCount(0)

    def __switchGUI__(self):
        """
        Function __switchGUI__ is capable of changing GUI based on number of connected inputs.
        """
        if(len(self.cluster) > len(self.reference)):
            self.optionsBox.setDisabled(1)
            #self.warning("Cluster dataset is greater than reference dataset. Please check the widget inputs.")
            QMessageBox.warning(None, "Invalid input dataset length", "Cluster dataset is longer than the reference dataset. Please check the widget inputs.", QMessageBox.Ok)
            return False
        if not self.reference and not self.cluster:
            self.optionsBox.setDisabled(1)
            return
        self.optionsBox.setDisabled(0)
        solo = True
        if self.reference and self.cluster:
            solo = False
        if solo:
            self.maxp.setDisabled(1)
            self.minf.setDisabled(0)
            for i in range(2, len(self.columns)):
                self.meshLV.hideColumn(i)
                self.sigTermsTable.hideColumn(i)

            self.sigTermsTable.setHorizontalHeaderLabels(["MeSH term", "frequency"])
            self.meshLV.setHeaderLabels(["MeSH term", "frequency"])
            self.ratio.setText("")
        else:
            self.maxp.setDisabled(0)
            self.minf.setDisabled(1)
            for i in range(0, len(self.columns)):
                self.meshLV.showColumn(i)
                self.sigTermsTable.showColumn(i)
            self.sigTermsTable.setHorizontalHeaderLabels(self.columns)
            self.meshLV.setHeaderLabels(self.columns)
            for i in range(0, len(self.columns)):
                self.meshLV.header().resizeSection(i, self.col_size[i])
                self.sigTermsTable.horizontalHeader().resizeSection(i, self.col_size[i])
            self.ratio.setText("ratio = %.4g" % self.mesh.ratio)

    def getReferenceData(self, data):
        """
        Function getReferenceData is executed once reference signal is connected to the widget.
        """
        if data:
            self.reference = data
            self.infoa.setText('%d reference examples' % len(data))
        else:
            self.reference = None
            self.infoa.setText('No reference data.')
            self.ref_att.setText('')

        if self.reference or self.cluster:
            self.__switchGUI__()
            self.__updateData__()
        else:
            self.__clearGUI__()

    def getClusterData(self, data):
        """
        Function getClusterData is executed once cluster signal is connected to the widget.
        """
        if data:
            self.cluster = data
            self.infob.setText('%d cluster examples' % len(data))
        else:
            self.cluster = None
            self.infob.setText('No cluster data.')
            self.clu_att.setText('')

        if self.reference or self.cluster:
            self.__switchGUI__()
            self.__updateData__()
        else:
            self.__clearGUI__()



#!/usr/bin/env python

from collections import defaultdict
from Orange.bio.obiGeneAtlas import run_simple_query
from Orange.bio.obiGeneSets import GeneSets, GeneSet
import time

def display_string(string):
    return string.capitalize().replace("_", " ")

regulation = "updown"
organism   = "Homo sapiens"
condition  = "organism_part"
org_code   = "9606"
max_pvalue = 1e-5

query = run_simple_query(regulation=regulation, organism=organism, condition=condition, start=0, rows=5) # Do a short pre-query to see how many genes there are
total_genes = query["totalResults"]

no_of_genes = 50 # 50 genes per query used, since the HTTP requests don't seem stable with more than 50 genes at a time.
no_of_pages = total_genes / no_of_genes  # Gene Expression Atlas API only permits access to [50,200] genes at a time.

print total_genes
#exit()

i=0
sets = defaultdict(list)

for start in range(no_of_pages+1):
    start = time.time()
    query = run_simple_query(regulation=regulation, organism=organism, condition=condition, start=start*no_of_genes, rows=no_of_genes)
    for result in query["results"]:
        i += 1
        print result["gene"]["name"] + "\t" + str(i) # For printing out gene names and the current number of genes during debugging
        for exp in result["expressions"]: 
            diff_exp = [e for e in exp["experiments"] if e["pvalue"] <= max_pvalue] # Use only genes that are significantly diff. expressed
            if diff_exp:
                try:
                    sets[exp["ef"], exp["efv"]].append(result["gene"]["name"]) # The Gene Expression Atlas entries are not consistent. 
                except:
                    sets[exp["efoTerm"], exp["efoId"]].append(result["gene"]["name"])
    print time.time() - start
        
gene_sets = []
for (ef, efv), genes in sets.items():
    ef_display = display_string(ef)
    gs = GeneSet(genes, "Diff. expressed in %s=%s." % (ef_display, efv), id=ef + ":" + efv,
                 description="Diff. expressed in %s=%s." % (ef_display, efv),
                 link="http://www.ebi.ac.uk/gxa/qrs?specie_0={organism}&gprop_0=&gnot_0=&gval_0=&fact_1=&fexp_1=UPDOWN&fmex_1=&fval_1=%22{efv}%22+&view=hm".format( \
                         organism = "+".join(organism.lower().split()), efv = "+".join(efv.lower().split())), organism=org_code, hierarchy=("Gene Expression Atlas", display_string(condition)))
    gene_sets.append(gs)

final_set = GeneSets(gene_sets)

print final_set
exit()

from Orange.bio.obiGeneSets import register
import Orange.utils.serverfiles as serverfiles
import sys

try:
    sf_server = serverfiles.ServerFiles(sys.argv[1], sys.argv[2])
except:
    print "argv[1] = username, argv[2] = password"
    exit()

set_split = final_set.split_by_hierarchy()

for s in set_split:
    register(s, sf_server)

print "Gene sets successfully registered..."



## Automatically adapted for numpy.oldnumeric Oct 04, 2007 by 

"""
<name>Approximate Profiles</name>
<description>Approximation of expression profiles by various kernel functions.</description>
<icon>icons/ApproximateProfiles.png</icon>
<priority>310</priority>
<contact>Peter Juvan (peter.juvan@fri.uni-lj.si)</contact>
<prototype>1</prototype>
"""

from __future__ import absolute_import

import numpy.oldnumeric as Numeric

from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWWidget import *

from .. import chipappx, chipstat
from .OWDataFiles import DataFiles

class OWApproxProfiles(OWWidget):
    settingsList  = ['kernel', 'kernelSize', 'useSignificance', 'alpha', 'commitOnChange']

    def __init__(self, parent=None, signalManager = None, name='Approximate Profiles'):
        self.callbackDeposit = [] # deposit for OWGUI callback functions
        OWWidget.__init__(self, parent, signalManager, name)

        self._data = None       # exampleTable
        self._dataN = None      # 2d numeric array
        self._chipdata = None       # [(dirname0, [et0, et1, ...]), ...]
        self._chipdataN = None      # 3d numeric array
        self.kernel = 0
        self.kernels = ["Orthogonal polynomials", "Trigonometric functions"]
        self.kernelSize = None
        self.kernelSizes = [    ["Const"] + map(lambda x: "degree <= %i"%x, range(1,99)),
                                ["Const", "cos x", "sin x"] + reduce(lambda x,y: x+[y[0],y[1]], map(lambda i: ("cos %ix"%i, "sin %ix"%i), range(2,99)), [])
                            ]
        self.useSignificance = 0
        self.alpha = 3
        self.alphas = [0.0001, 0.001, 0.01, 0.05, 0.1, 0.2, 0.5]
        self.commitOnChange = 1
               
        # Settings
        self.loadSettings()

        # GUI: info, comboKernelSize, cbUseSignificance, vboxSignificance, commitBtn
        # info
        box = QVGroupBox("Info", self.controlArea)
        self.infoa = QLabel("No examples on input", box)
        OWGUI.separator(box,250)
        self.infob = QLabel("No structured data on input", box)
        OWGUI.separator(self.controlArea)

        # kernel selection
        box = QVGroupBox("Kernel functions", self.controlArea)
        OWGUI.comboBox(box, self, "kernel", items=self.kernels, callback=self.kernelChange)
        OWGUI.separator(self.controlArea)

        # kernel settings
        box = QVGroupBox("Kernel settings", self.controlArea)
        self.comboKernelSize = OWGUI.comboBox(box, self, "kernelSize", callback=self.kernelSizeChange, label="Number of kernel functions", labelWidth=135, orientation="horizontal", valueType=int)
        self.comboKernelSize.setDisabled(1)
        self.cbUseSignificance = OWGUI.checkBox(box, self, "useSignificance", "Significance of coefficients (F-statistics)", callback=self.useSignificanceChange, tooltip="Use kernels with coefficients significantly different from 0.")
        self.vboxSignificance = QVBox(box)
        OWGUI.comboBox(self.vboxSignificance, self, "alpha", items = self.alphas, callback=self.alphaChange, label="p <", labelWidth=20, orientation="horizontal")
        OWGUI.separator(self.controlArea)

        # output
        box = QVGroupBox("Output", self.controlArea)
        OWGUI.checkBox(box, self, 'commitOnChange', 'Commit data on selection change')
        self.commitBtn = OWGUI.button(box, self, "Commit", callback=self.senddata, disabled=1)

        self.inputs = [("Examples", ExampleTable, self.data), ("Structured Data", DataFiles, self.chipdata)]
        self.outputs = [("Approximated Examples", ExampleTable, Default), ("Approximation Coefficients", ExampleTable), ("Approximated Structured Data", DataFiles, Default), ("Structured Approximation Coefficients", DataFiles)]
        self.resize(200,100)

    def kernelChange(self):
        numPoints = self._getMinDataShape1()
        if self.kernel == 0 and self.kernelSize < numPoints-1:
            self.cbUseSignificance.setEnabled(1)
            self.vboxSignificance.setEnabled(1)
        else:
            self.cbUseSignificance.setDisabled(1)
            self.vboxSignificance.setDisabled(1)
        self.setGuiCommonExpChip()
        if self.commitOnChange:
            self.senddata()

    def kernelSizeChange(self):
        numPoints = self._getMinDataShape1()
        if self.kernel == 0:
            if self.kernelSize in [0, numPoints-1]:
                self.cbUseSignificance.setDisabled(1)
                self.vboxSignificance.setDisabled(1)
            else:
                self.cbUseSignificance.setEnabled(1)
                self.vboxSignificance.setEnabled(1)
        if self.commitOnChange:
            self.senddata()

    def alphaChange(self):
        if self.commitOnChange:
            self.senddata()

    def useSignificanceChange(self):
        if self.useSignificance:
            self.vboxSignificance.setEnabled(1)
        else:
            self.vboxSignificance.setDisabled(1)
        if self.commitOnChange:
            self.senddata()

    def _getMinDataShape1(self):            
        numPoints = 1e9 # number of attributes (if both inputs present: take min.)
        if self._dataN != None: numPoints = min(numPoints, self._dataN.shape[1])
        if self._chipdataN != None: numPoints = min(numPoints, self._chipdataN.shape[1])
        if numPoints == 1e9: numPoints = 0
        return numPoints

    def setGuiCommonExpChip(self):
        """Sets GUI features that depend on both inputs, e.g. max. values of sliders, commit button.
        """
        numPoints = self._getMinDataShape1()
        # set up comboKernelSize
        self.comboKernelSize.clear()
        # print "numPoints:", numPoints
        if numPoints > 0:
            for i in range(0,numPoints):
                self.comboKernelSize.insertItem(self.kernelSizes[self.kernel][i])
            if self.kernelSize==None or self.kernelSize >= numPoints:
                self.kernelSize = max([int((numPoints)/2)-2,0])
            self.comboKernelSize.setCurrentItem(self.kernelSize)
            self.comboKernelSize.setEnabled(1)
            self.commitBtn.setEnabled(1)
        else:
            self.comboKernelSize.setDisabled(1)
            #self.kernelSize = None
            self.commitBtn.setDisabled(1)

    def data(self, data):
        if data != None:
            self._data = data
            self._dataN = chipstat.orng2ma(data)
            self.infoa.setText("Examples: %i profiles on %i points" % (self._dataN.shape[0], self._dataN.shape[1]))
        else:
            self._data = None
            self._dataN = None
            self.infoa.setText("No examples on input")
        self.setGuiCommonExpChip()
        if self.commitOnChange:
            self.senddata(1)

    def chipdata(self, data):
        if data != None:
            self._chipdata = data
            shp = [0,0,0]
            shp[0] = len(data[0][1][0])
            shp[1] = len(data[0][1][0].domain.attributes)
            shp[2] = reduce(lambda x,y: x+len(y[1]), data, 0)
            self._chipdataN = Numeric.zeros(shp, Numeric.Float)
            idx = 0
            for (name, etList) in data:
                for et in etList:
                    self._chipdataN[:,:,idx] = Numeric.asarray(chipstat.orng2ma(et))
                    idx += 1
            self.infob.setText("Structured Data: %i data files with %i profiles on %i points" % (shp[2], shp[0], shp[1]))
        else:
            self._chipdata = None
            self._chipdataN = None
            self.infob.setText("No structured data on input")
        self.setGuiCommonExpChip()
        if self.commitOnChange:
            self.senddata(2)

    def senddata(self, outputSelector=0):
        """outputSelector = [0: examples + chip data | 1: examples | 2: chip data]
        """
        assert outputSelector in [0,1,2]
        # progress bar settings: 1: examples, 2: chip data
        steps = 0
        if outputSelector in [0,1] and self._dataN != None:
            steps += 3                                  # 1 step for approximator, 1 step for coefficients, 1 step for curves
        if outputSelector in [0,2] and self._chipdataN != None:
            steps += (1 + 2*self._chipdataN.shape[2])   # 1 step for approximator, for each dataset 2 steps for coeficients and curves
        if steps == 0: steps = 1
        pbStep = 100./steps
        self.progressBarInit()
        if outputSelector in [0,1]:
            self._sendexampledata(pbStep)
        if outputSelector in [0,2]:
            self._sendchipdata(pbStep)
        self.progressBarFinished()

    def _sendexampledata(self, pbStep):
        if self._dataN != None:
            # approximation
            if self.kernel == 0:
                appx = chipappx.ApproxOrthPolyBasis(range(self._dataN.shape[1]), self.kernelSize+1)
                self.progressBarAdvance(pbStep)
                if self.useSignificance and self.cbUseSignificance.isEnabled():
                    coef = appx.getAppxCoef2d_significant(self._dataN, self.kernelSize+1, self.alphas[self.alpha])
                else:
                    coef = appx.getAppxCoef(self._dataN)
                self.progressBarAdvance(pbStep)
                curve = appx.getAppxCurve(coef)
                self.progressBarAdvance(pbStep)
            elif self.kernel == 1:
                appx = chipappx.TrigonomerticBasis(self._dataN.shape[1], self.kernelSize+1)
                self.progressBarAdvance(pbStep)
                # 2007-10-11: trigonometric functions do not use getAppxCoef2d_significant
                #if self.useSignificance and self.cbUseSignificance.isEnabled():
                #    coef = appx.getAppxCoef2d_significant(self._dataN, self.kernelSize+1, self.alphas[self.alpha])
                #else:
                coef = appx.getAppxCoef(self._dataN)
##                print self._dataN
##                print "coef", coef
                self.progressBarAdvance(pbStep)
                curve = appx.getAppxCurve(coef)
##                print "curve", curve
                self.progressBarAdvance(pbStep)
            # domain with class and metas
            if self._data.domain.classVar != None:
                domainClassMetas = orange.Domain([self._data.domain.classVar])
            else:
                domainClassMetas = orange.Domain([])
            domainClassMetas.addmetas(self._data.domain.getmetas())
            # exampleTable with class and metas
            etClassMetas = orange.ExampleTable(domainClassMetas, self._data)
            # appx. curve
            etCurve = orange.ExampleTable(orange.Domain(self._data.domain.attributes, None), curve.tolist())
            etCurve = orange.ExampleTable([etCurve, etClassMetas])
            self.send("Approximated Examples", etCurve)
            # appx. coefficients
            domainCoef = orange.Domain(map(lambda x: orange.FloatVariable("C%i" % x), range(coef.shape[1])), None)
            etCoef = orange.ExampleTable(domainCoef, coef.tolist())
            etCoef = orange.ExampleTable([etCoef, etClassMetas])
            self.send("Approximation Coefficients", etCoef)
        else:
            self.send("Approximated Examples", None)
            self.send("Approximation Coefficients", None)

    def _sendchipdata(self, pbStep):
        if self._chipdataN != None:
            # approximation
            coefs = Numeric.zeros((self._chipdataN.shape[0], self.kernelSize+1, self._chipdataN.shape[2]), Numeric.Float)
            curves = Numeric.zeros(self._chipdataN.shape, Numeric.Float)
            if self.kernel == 0:
                appx = chipappx.ApproxOrthPolyBasis(range(self._chipdataN.shape[1]), self.kernelSize+1)
                self.progressBarAdvance(pbStep)
                for idx2 in range(self._chipdataN.shape[2]):
                    if self.useSignificance and self.cbUseSignificance.isEnabled():
                        coefs[:,:,idx2] = appx.getAppxCoef2d_significant(self._chipdataN[:,:,idx2], self.kernelSize+1, self.alphas[self.alpha])
                    else:
                        coefs[:,:,idx2] = appx.getAppxCoef(self._chipdataN[:,:,idx2])
                    self.progressBarAdvance(pbStep)
                    curves[:,:,idx2] = appx.getAppxCurve(coefs[:,:,idx2])
                    self.progressBarAdvance(pbStep)
            elif self.kernel == 1:
                appx = chipappx.TrigonomerticBasis(self._chipdataN.shape[1], self.kernelSize+1)
                self.progressBarAdvance(pbStep)
                for idx2 in range(self._chipdataN.shape[2]):
                    # 2007-10-11: trigonometric functions do not use getAppxCoef2d_significant
                    #if self.useSignificance and self.cbUseSignificance.isEnabled():
                    #    coefs[:,:,idx2] = appx.getAppxCoef2d_significant(self._chipdataN[:,:,idx2], self.kernelSize+1, self.alphas[self.alpha])
                    #else:
                    coefs[:,:,idx2] = appx.getAppxCoef(self._chipdataN[:,:,idx2])
                    self.progressBarAdvance(pbStep)
                    curves[:,:,idx2] = appx.getAppxCurve(coefs[:,:,idx2])
                    self.progressBarAdvance(pbStep)
            chipcoefNew = []    # [(dirname0, [etCoef0, etCoef1, ...]), ...]
            chipcurvesNew = []  # [(dirname0, [etCurves0, etCurves1, ...]), ...]
            idxTotal = 0
            for (dirname, etList) in self._chipdata:
                etCoefListNew = []
                etCurvesListNew = []
                for et in etList:
                    # domain with class and metas
                    if et.domain.classVar != None:
                        domainClassMetas = orange.Domain([et.domain.classVar])
                    else:
                        domainClassMetas = orange.Domain([])
                    domainClassMetas.addmetas(et.domain.getmetas())
                    # exampleTable with class and metas
                    etClassMetas = orange.ExampleTable(domainClassMetas, et)
                    # appx. coefficients
                    domainCoef = orange.Domain(map(lambda x: orange.FloatVariable("C%i" % x), range(coefs.shape[1])), None)
                    etCoef = orange.ExampleTable(domainCoef, coefs[:,:,idxTotal].tolist())
                    etCoefListNew.append(orange.ExampleTable([etCoef, etClassMetas]))
                    etCoefListNew[-1].name = et.name
                    # appx. curve
                    etCurve = orange.ExampleTable(orange.Domain(et.domain.attributes, None), curves[:,:,idxTotal].tolist())
                    etCurvesListNew.append(orange.ExampleTable([etCurve, etClassMetas]))
                    etCurvesListNew[-1].name = et.name
                    idxTotal += 1
                chipcoefNew.append((dirname, etCoefListNew))
                chipcurvesNew.append((dirname, etCurvesListNew))
            self.send("Approximated Structured Data", chipcurvesNew)
            self.send("Structured Approximation Coefficients", chipcoefNew)
        else:
            self.send("Approximated Structured Data", None)
            self.send("Structured Approximation Coefficients", None)


if __name__=="__main__":
    a=QApplication(sys.argv)
    ow=OWApproxProfiles()
    a.setMainWidget(ow)
    ow.show()
    ow.data(orange.ExampleTable("meanExpr_ann_pkaC.tab"))
    a.exec_loop()
    ow.saveSettings()



"""
<name>Differential expression</name>
<description>Gene differential expression scoring and selection.</description>
<priority>1010</priority>
<icon>icons/GeneSelection.svg</icon>
"""

from __future__ import absolute_import, with_statement

from collections import defaultdict
from functools import wraps
from operator import add

import numpy as np
import numpy.ma as ma

import orange
from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWGraph import *
from Orange.OrangeWidgets.OWGraphTools import PolygonCurve
from Orange.OrangeWidgets.OWHist import OWInteractiveHist
from Orange.OrangeWidgets.OWToolbars import ZoomSelectToolbar
from Orange.OrangeWidgets.OWWidget import *

from ..obiExpression import *


NAME = "Differential Expression"
DESCRIPTION = "Gene selection by differential expression analysis."
ICON = "icons/GeneSelection.svg"
PRIORITY = 1010

INPUTS = [("Examples", Orange.data.Table, "set_data")]
OUTPUTS = [("Example table with selected genes", Orange.data.Table, Default),
           ("Example table with remaining genes", Orange.data.Table),
           ("Selected genes", Orange.data.Table)]

REPLACES = ["_bioinformatics.widgets.OWFeatureSelection.OWFeatureSelection"]


class ExpressionSignificance_TTest_PValue(ExpressionSignificance_TTest):
    def __call__(self, *args, **kwargs):
        return [(key, pval) for key, (t, pval) in \
                ExpressionSignificance_TTest.__call__(self, *args, **kwargs)]


class ExpressionSignificance_TTest_T(ExpressionSignificance_TTest):
    def __call__(self, *args, **kwargs):
        return [(key, t) for key, (t, pval) in \
                ExpressionSignificance_TTest.__call__(self, *args, **kwargs)]


class ExpressionSignificance_ANOVA_PValue(ExpressionSignificance_ANOVA):
    def __call__(self, *args, **kwargs):
        return [(key, pval) for key, (t, pval) in \
                ExpressionSignificance_ANOVA.__call__(self, *args, **kwargs)]


class ExpressionSignificance_ANOVA_F(ExpressionSignificance_ANOVA):
    def __call__(self, *args, **kwargs):
        return [(key, f) for key, (f, pval) in \
                ExpressionSignificance_ANOVA.__call__(self, *args, **kwargs)]


class ExpressionSignificance_Log2FoldChange(ExpressionSignificance_FoldChange):
    def __call__(self, *args, **kwargs):
        return [(key, math.log(fold, 2.0) if fold > 1e-300 and fold < 1e300 else 0.0) \
                for key, fold in ExpressionSignificance_FoldChange.__call__(self, *args, **kwargs)]


class ExpressionSignigicance_MannWhitneyu_U(ExpressionSignificance_MannWhitneyu):
    def __call__(self, *args, **kwargs):
        return [(key, u) for key, (u, p_val) in \
                ExpressionSignificance_MannWhitneyu.__call__(self, *args, **kwargs)]


class ScoreHist(OWInteractiveHist):
    def __init__(self, master, parent=None, type="hiTail"):
        OWInteractiveHist.__init__(self, parent, type=type)
        self.master = master
        self.setAxisTitle(QwtPlot.xBottom, "Score")
        self.setAxisTitle(QwtPlot.yLeft, "Frequency")
        self.activateSelection()

    def setBoundary(self, low, hi):
        OWInteractiveHist.setBoundary(self, low, hi)
        self.master.on_boundary_change(low, hi)


def disable_controls(method):
    """ Disable the widget's control area during the duration of this call.
    """
    @wraps(method)
    def f(self, *args, **kwargs):
        self.controlArea.setDisabled(True)
        qApp.processEvents()
        try:
            return method(self, *args, **kwargs)
        finally:
            self.controlArea.setDisabled(False)
    return f

from Orange.orng.orngDataCaching import data_hints

from .OWGenotypeDistances import SetContextHandler
from .OWVulcanoPlot import LabelSelectionWidget


class OWFeatureSelection(OWWidget):
    settingsList = [
        "method_index", "dataLabelIndex", "compute_null",
        "permutations_count", "selectPValue", "auto_commit",
        "thresholds"]

    contextHandlers = {
        "Data": DomainContextHandler("Data", ["genes_in_columns"]),
        "TargetSelection":
            SetContextHandler("TargetSelection", findImperfect=False)
    }

    def __init__(self, parent=None, signalManager=None, name="Gene selection"):
        OWWidget.__init__(self, parent, signalManager, name, wantGraph=True, showSaveGraph=True)
        self.inputs = [("Examples", ExampleTable, self.set_data)]
        self.outputs = [("Example table with selected genes", ExampleTable), ("Example table with remaining genes", ExampleTable), ("Selected genes", ExampleTable)]

        self.method_index = 0
        self.genes_in_columns = False
        self.compute_null = False
        self.permutations_count = 10
        self.auto_commit = False
        self.selectNBest = 20
        self.selectPValue = 0.01
        self.data_changed_flag = False
        self.add_scores_to_output = True
        self.thresholds = {
            "fold change": (0.5, 2.),
            "log2 fold change": (-1, 1),
            "t-test": (-2, 2),
            "t-test p-value": (0.01, 0.01),
        }

        self.oneTailTestHi = oneTailTestHi = lambda array, low, hi: array >= hi
        self.oneTailTestLow = oneTailTestLow = lambda array, low, hi: array <= low
        self.twoTailTest = twoTailTest = lambda array, low, hi: (array >= hi) | (array <= low)
        self.middleTest = middleTest = lambda array, low, hi: (array <= hi) | (array >= low)
        
        self.histType = {oneTailTestHi:"hiTail", oneTailTestLow:"lowTail", twoTailTest:"twoTail", middleTest:"middle"}

        # [(name, func, tail test, two sample test), ...]
        self.score_methods = [("fold change", ExpressionSignificance_FoldChange, twoTailTest, True),
                             ("log2 fold change", ExpressionSignificance_Log2FoldChange, twoTailTest, True),
                             ("t-test", ExpressionSignificance_TTest_T, twoTailTest, True),
                             ("t-test p-value", ExpressionSignificance_TTest_PValue, oneTailTestLow, True),
                             ("anova", ExpressionSignificance_ANOVA_F, oneTailTestHi, False),
                             ("anova p-value", ExpressionSignificance_ANOVA_PValue, oneTailTestLow, False),
                             ("signal to noise ratio", ExpressionSignificance_SignalToNoise, twoTailTest, True),
                             ("info gain", ExpressionSignificance_Info, oneTailTestHi, True),
                             ("chi-square", ExpressionSignificance_ChiSquare, oneTailTestHi, True),
                             ("mann-whitney", ExpressionSignigicance_MannWhitneyu_U, oneTailTestLow, True)]

        boxHistogram = OWGUI.widgetBox(self.mainArea)
        self.histogram = ScoreHist(self, boxHistogram)
        boxHistogram.layout().addWidget(self.histogram)
        self.histogram.show()
        
        box = OWGUI.widgetBox(self.controlArea, "Info")
        self.dataInfoLabel = OWGUI.widgetLabel(box, "\n\n")
        self.dataInfoLabel.setWordWrap(True)
        self.selectedInfoLabel = OWGUI.widgetLabel(box, "")

        box1 = OWGUI.widgetBox(self.controlArea, "Scoring Method")
        self.testRadioBox = OWGUI.comboBox(box1, self, "method_index",
                                    items=[sm[0] for sm in self.score_methods],
                                    callback=[self.on_scoring_method_changed, self.update_scores])
        
        box = OWGUI.widgetBox(self.controlArea, "Target Labels")
        self.label_selection_widget = LabelSelectionWidget(self)
        self.label_selection_widget.setMaximumHeight(150)
        box.layout().addWidget(self.label_selection_widget)
        self.connect(self.label_selection_widget,
                     SIGNAL("selection_changed()"),
                     self.on_target_changed)
        
        self.connect(self.label_selection_widget,
                     SIGNAL("label_activated(int)"),
                     self.on_label_activated)
        
        self.genes_in_columns_check = OWGUI.checkBox(box, self, "genes_in_columns",
                                                  "Genes in columns",
                                                  callback=self.on_genes_in_columns_change)

        box = OWGUI.widgetBox(self.controlArea, "Selection")
        box.layout().setSpacing(0)

        self.upperBoundarySpin = OWGUI.doubleSpin(box, self, "histogram.upperBoundary",
                                                  min=-1e6, max=1e6, step= 1e-6,
                                                  label="Upper threshold:", 
                                                  callback=self.update_boundary, 
                                                  callbackOnReturn=True)
        
        self.lowerBoundarySpin = OWGUI.doubleSpin(box, self, "histogram.lowerBoundary", 
                                                  min=-1e6, max=1e6, step= 1e-6, 
                                                  label="Lower threshold:", 
                                                  callback=self.update_boundary, 
                                                  callbackOnReturn=True)
        
        check = OWGUI.checkBox(box, self, "compute_null", "Compute null distribution",
                               callback=self.update_scores)

        check.disables.append(OWGUI.spin(box, self, "permutations_count", min=1, max=10, 
                                         label="Permutations:", callback=self.update_scores, 
                                         callbackOnReturn=True))

        box1 = OWGUI.widgetBox(box, orientation='horizontal')
        check.disables.append(OWGUI.doubleSpin(box1, self, "selectPValue", 
                                               min=2e-7, max=1.0, step=1e-7, 
                                               label="P-value:"))
        check.disables.append(OWGUI.button(box1, self, "Select", callback=self.select_p_best))
        check.makeConsistent()

        box1 = OWGUI.widgetBox(box, orientation='horizontal')
        OWGUI.spin(box1, self, "selectNBest", 0, 10000, step=1, label="Best Ranked:")
        OWGUI.button(box1, self, "Select", callback=self.select_n_best)

        box = OWGUI.widgetBox(self.controlArea, "Output")
        b = OWGUI.button(box, self, "&Commit", callback=self.commit)
        cb = OWGUI.checkBox(box, self, "auto_commit", "Commit on change")
        OWGUI.setStopper(self, b, cb, "data_changed_flag", self.commit)
        OWGUI.checkBox(box, self, "add_scores_to_output", "Add gene scores to output",
                       callback=self.commit_if) 
        
        OWGUI.rubber(self.controlArea)

        self.connect(self.graphButton, SIGNAL("clicked()"), self.histogram.saveToFile)
        
        self.loadSettings()

        self.data = None
        self.discData = None
        self.scoreCache = {}
        self.nullDistCache = {}
        self.cuts = {}
        self.null_dist = []
        self.targets = []
        self.scores = {}
        self.genes_in_columns = True
        self.target_selections = None
        
        self.on_scoring_method_changed()
        
        self.resize(800, 600)
        
    def clear(self):
        """ Clear widget.
        """
        self.scoreCache = {}
        self.nullDistCache = {}
        self.discData = None
        self.data = None
        self.attribute_targets = []
        self.class_targets = []
        self.label_selection_widget.clear()
        self.clear_plot()
        
    def clear_plot(self):
        """ Clear the histogram plot.
        """
        self.histogram.removeDrawingCurves()
        self.histogram.clear()
        self.histogram.replot()
        
    def init_from_data(self, data):
        """ Init widget state from the data.
        """
        if data:
            items = [attr.attributes.items() for attr in data.domain.attributes]
            items = reduce(add, items, [])
            
            targets = defaultdict(set)
            for label, value in items:
                targets[label].add(value)
                
            targets = [(key, sorted(vals)) for key, vals in targets.items() \
                       if len(vals) >= 2]
            self.attribute_targets = targets
            
            class_var = data.domain.class_var
            if class_var and isinstance(class_var, orange.EnumVariable):
                targets = [(class_var.name,
                            list(class_var.values))]
                self.class_targets = targets
            else:
                self.class_targets = []
            
    def update_targets_widget(self):
        """ Update the contents of the targets widget.
        """
        if self.data:
            if self.genes_in_columns:
                targets = self.attribute_targets
            elif self.data.domain.classVar:
                targets = self.class_targets
            else:
                targets = []
        else:
            targets = []
            
        self.label_selection_widget.clear()
        self.label_selection_widget.set_labels(targets)
        self.data_labels = targets

    def set_data(self, data):
        self.closeContext("Data")
        self.closeContext("TargetSelection")
        self.error([0, 1])
        self.warning(0)
        self.clear()
        self.genes_in_columns_check.setEnabled(True)
        self.data = data
        self.init_from_data(data)

        if self.data:
            self.genes_in_columns = not data_hints.get_hint(data, "genesinrows", False)
            self.openContext("Data", data)

            # If only attr. labels or only class values then disable
            # the 'Genes in columns' control
            if not self.attribute_targets or not self.class_targets:
                self.genes_in_columns_check.setEnabled(False)
                self.genes_in_columns = bool(self.attribute_targets)

        self.update_targets_widget()

        if self.data is not None  and \
                not (self.attribute_targets or self.class_targets):
            # If both attr. labels and classes are missing, show an error
            self.error(1, "Cannot compute gene scores! Differential expression widget "
                          "requires a data-set with a discrete class variable "
                          "or attribute labels!")
            self.data = None

        if self.data:
            # Load context selection
            items = [(label, v) for label, values in self.data_labels for v in values]

            self.target_selections = [values[:1] for _, values in self.data_labels]
            label, values = self.data_labels[0]
            self.current_target_selection = label, values[:1] # Default selections

            self.openContext("TargetSelection", set(items)) # Load selections from context
            self.label_selection_widget.set_selection(*self.current_target_selection)

        if not self.data:
            self.send("Example table with selected genes", None)
            self.send("Example table with remaining genes", None)
            self.send("Selected genes", None)

    def set_targets(self, targets):
        """ Set the target groups for score computation.
        """
        self.targets = targets
        self.update_scores()
    
    def compute_scores(self, data, score_func, use_attribute_labels,
                       target=None, advance=lambda: None):
        score_func = score_func(data, use_attribute_labels)
        advance()
        score = score_func(target=target)
        score = [(key, val) for key, val in score if val is not ma.masked]
        return score
    
    def compute_null_distribution(self, data, score_func, use_attributes,
                                  target=None, perm_count=10, advance=lambda: None):
        score_func = score_func(data, use_attributes)
        dist = score_func.null_distribution(perm_count, target, advance=advance)
        return [score for run in dist for k, score in run if score is not ma.masked]
            
    @disable_controls
    def update_scores(self):
        """ Compute the scores and update the histogram.
        """
        self.clear_plot()
        self.error(0)
        label, values = self.current_target_selection
        if not self.data or label is None:
            return
        _, score_func, _, two_sample_test = self.score_methods[self.method_index]
        if two_sample_test:
            target = self.targets
            score_target = set(target)
            ind1, ind2 = score_func(self.data, self.genes_in_columns).test_indices(score_target)
            if not len(ind1) or not len(ind2):
                self.error(0, "Target labels most exclude/include at least one value.")
                return
            
        else: # ANOVA should use all labels. 
            target = dict(self.data_labels)[label]
            if self.genes_in_columns:
                target = [(label, t) for t in target]
            score_target = target
#            indices = score_func(self.data, self.genes_in_columns).test_indices(score_target)
            # TODO: Check that each label has more than one measurement, raise warning otherwise.
         
        pb = OWGUI.ProgressBar(self, 4 + self.permutations_count if self.compute_null else 3)
        self.scores = dict(self.compute_scores(self.data, score_func,
                    self.genes_in_columns, score_target, advance=pb.advance))
        pb.advance()
        if self.compute_null:
            self.null_dist = self.compute_null_distribution(self.data,
                        score_func, self.genes_in_columns, score_target,
                        self.permutations_count, advance=pb.advance)
        else:
            self.null_dist = []
        pb.advance()
        htype = self.histType[self.score_methods[self.method_index][2]]
        score_type = self.score_methods[self.method_index][0]
        self.histogram.type = htype
        if self.scores:
            self.histogram.setValues(self.scores.values())
            low, high = self.thresholds.get(score_type, (float("-inf"), float("inf")))
            minx, maxx = self.histogram.minx, self.histogram.maxx
            low, high = max(low, minx), min(high, maxx)

            if htype == "hiTail":
                low = high
            if htype == "lowTail":
                high = low

            self.histogram.setBoundary(low, high)

            if self.compute_null and self.null_dist:
                nullY, nullX = numpy.histogram(self.null_dist, bins=self.histogram.xData)
                nullY = nullY / self.permutations_count
                self.histogram.nullCurve = self.histogram.addCurve(
                    "nullCurve", Qt.black, Qt.black, 6,
                    symbol=QwtSymbol.NoSymbol, style=QwtPlotCurve.Steps,
                    xData=nullX, yData=nullY)

                minx = min(min(nullX), minx)
                maxx = max(max(nullX), maxx)
                miny = min(min(nullY), self.histogram.miny)
                maxy = max(max(nullY), self.histogram.maxy)
                spanx, spany = maxx - minx, maxy - miny
                self.histogram.setAxisScale(
                    QwtPlot.xBottom, minx - 0.05 * spanx, maxx + 0.05 * spanx)
                self.histogram.setAxisScale(
                    QwtPlot.yLeft, miny - 0.05 * spany, maxy + 0.05 * spany)

            state = dict(hiTail=(False, True), lowTail=(True, False), twoTail=(True, True))
            for spin, visible in zip((self.upperBoundarySpin, self.lowerBoundarySpin), state[self.histogram.type]):
                spin.setVisible(visible)

            # If this is a two sample test add markers to the left and right
            # plot indicating which target group is over-expressed in that
            # part
            if self.method_index in [0, 2, 6]:
                if self.method_index == 0: ## fold change is centered on 1.0
                    x1, y1 = (self.histogram.minx + 1) / 2 , self.histogram.maxy
                    x2, y2 = (self.histogram.maxx + 1) / 2 , self.histogram.maxy
                else:
                    x1, y1 = (self.histogram.minx) / 2 , self.histogram.maxy
                    x2, y2 = (self.histogram.maxx) / 2 , self.histogram.maxy
                if self.genes_in_columns:
                    label = target[0][0]
                    target_values = [t[1] for t in target]
                    values = dict(self.data_labels)[label]
                else:
                    target_values = target
                    values = self.data_labels[0][1]
                    
                left = ", ".join(v for v in values if v not in target_values)
                right = ", ".join(v for v in values if v in target_values)
                    
                self.histogram.addMarker(left, x1, y1)
                self.histogram.addMarker(right, x2, y2)
            self.warning(0)
        else:
            self.warning(0, "No scores obtained.")
        self.histogram.replot()
        pb.advance()
        pb.finish()
        self.update_data_info_label()

    def on_boundary_change(self, low, high):
        stype = self.score_methods[self.method_index][0]
        self.thresholds[stype] = (low, high)
        self.update_selected_info_label(low, high)
        self.commit_if()

    def update_data_info_label(self):
        if self.data:
            samples, genes = len(self.data), len(self.data.domain.attributes)
            if self.genes_in_columns:
                samples, genes = genes, samples
                target_labels = [t[1] for t in self.targets]
            else:
                target_labels = self.targets
            text = "%i samples, %i genes\n" % (samples, genes)
            text += "Sample target: '%s'" % (",".join(target_labels))
        else:
            text = "No data on input\n"
        self.dataInfoLabel.setText(text)

    def update_selected_info_label(self, cutOffLower=0, cutOffUpper=0):
        self.cuts[self.method_index] = (cutOffLower, cutOffUpper)
        if self.data:
            scores = np.array(self.scores.values())
            test = self.score_methods[self.method_index][2]
            self.selectedInfoLabel.setText("%i selected genes" % len(np.nonzero(test(scores, cutOffLower, cutOffUpper))[0]))
        else:
            self.selectedInfoLabel.setText("0 selected genes")

    def select_n_best(self):
        scores = self.scores.items()
        scores.sort(lambda a,b:cmp(a[1], b[1]))
        if not scores:
            return
        if self.score_methods[self.method_index][2]==self.oneTailTestHi:
            scores = scores[-max(self.selectNBest, 1):]
            self.histogram.setBoundary(scores[0][1], scores[0][1])
        elif self.score_methods[self.method_index][2]==self.oneTailTestLow:
            scores = scores[:max(self.selectNBest,1)]
            self.histogram.setBoundary(scores[-1][1], scores[-1][1])
        else:
            scoresHi = scores[-max(min(self.selectNBest, len(scores)/2), 1):]
            scoresLo = scores[:max(min(self.selectNBest, len(scores)/2), 1)]
            scores = [(abs(score), 1) for attr, score in scoresHi] + [(abs(score), -1) for attr, score in scoresLo]
            if self.score_methods[self.method_index][0]=="fold change": ## comparing fold change on a logaritmic scale
                scores =  [(abs(math.log(max(min(score, 1e300), 1e-300), 2.0)), sign) for score, sign in scores]
            scores.sort()
            scores = scores[-max(self.selectNBest, 1):]
            countHi = len([score for score, sign in scores if sign==1])
            countLo = len([score for score, sign in scores if sign==-1])
            cutHi = scoresHi[-countHi][1] if countHi else scoresHi[-1][1] + 1e-7
            cutLo = scoresLo[countLo-1][1] if countLo else scoresLo[0][1] - 1e-7
            self.histogram.setBoundary(cutLo, cutHi)

    def update_boundary(self):
        if self.data != None and self.scores.items():
            if self.score_methods[self.method_index][2]==self.oneTailTestHi:
                self.histogram.setBoundary(self.histogram.lowerBoundary, self.histogram.lowerBoundary)
            elif self.score_methods[self.method_index][2]==self.oneTailTestLow:
                self.histogram.setBoundary(self.histogram.upperBoundary, self.histogram.upperBoundary)
            else:
                self.histogram.setBoundary(self.histogram.lowerBoundary, self.histogram.upperBoundary)

    def select_p_best(self):
        if not self.null_dist:
            return
        nullDist = sorted(self.null_dist)
        test = self.score_methods[self.method_index][2]
        count = min(int(len(nullDist)*self.selectPValue), len(nullDist))
        if test == self.oneTailTestHi:
            cut = nullDist[-count] if count else nullDist[-1] # + 1e-7
            self.histogram.setBoundary(cut, cut)
        elif test == self.oneTailTestLow:
            cut = nullDist[count - 1] if count else nullDist[0] # - 1e-7
            self.histogram.setBoundary(cut, cut)
        elif count:
            scoresHi = nullDist[-count:]
            scoresLo = nullDist[:count]
            scores = [(abs(score), 1) for score in scoresHi] + [(abs(score), -1) for score in scoresLo]
            if self.score_methods[self.method_index][0] == "fold change": ## fold change is on a logaritmic scale
                scores =  [(abs(math.log(max(min(score, 1e300), 1e-300), 2.0)), sign) for score, sign in scores]
            scores = sorted(scores)[-count:]
            countHi = len([score for score, sign in scores if sign==1])
            countLo = len([score for score, sign in scores if sign==-1])
            cutHi = scoresHi[-countHi] if countHi else scoresHi[-1] + 1e-7
            cutLo = scoresLo[countLo-1] if countLo else scoresLo[0] - 1e-7
            self.histogram.setBoundary(cutLo, cutHi)
        else:
            self.histogram.setBoundary(nullDist[0] - 1e-7, nullDist[-1] + 1e-7)
            

    def commit_if(self):
        if self.auto_commit:
            self.commit()
        else:
            self.data_changed_flag = True
        
    def commit(self):
        if not self.data or not self.scores:
            return
        test = self.score_methods[self.method_index][2]
        
        cutOffUpper = self.histogram.upperBoundary
        cutOffLower = self.histogram.lowerBoundary
        
        scores = np.array(self.scores.items())
        scores[:, 1] = test(np.array(scores[:, 1], dtype=float), cutOffLower, cutOffUpper)
        selected = set([key for key, test in scores if test])
        remaining = set([key for key, test in scores if not test])
        if self.data and self.genes_in_columns:
            selected = sorted(selected)
            if selected:
                newdata = orange.ExampleTable(
                    orange.Domain(self.data.domain),
                    [self.data[int(i)] for i in selected],
                    name=self.data.name
                )
            else:
                newdata = None
            if self.add_scores_to_output:
                score_attr = orange.FloatVariable(self.score_methods[self.method_index][0])
                mid = orange.newmetaid()
                
            if self.add_scores_to_output and newdata is not None:
                newdata.domain.addmeta(mid, score_attr)
                for ex, key in zip(newdata, selected):
                    ex[mid] = self.scores[key]
                    
            self.send("Example table with selected genes", newdata)
            
            remaining = sorted(remaining)
            if remaining:
                newdata = orange.ExampleTable(
                    orange.Domain(self.data.domain),
                    [self.data[int(i)] for i in remaining],
                    name=self.data.name
                )
            else:
                newdata = None
            
            if self.add_scores_to_output and newdata is not None:
                newdata.domain.addmeta(mid, score_attr)
                for ex, key in zip(newdata, remaining):
                    ex[mid] = self.scores[key]
                    
            self.send("Example table with remaining genes", newdata)
            
        elif self.data and not self.genes_in_columns:
            method_name = self.score_methods[self.method_index][0]
            selected_attrs = [attr for attr in self.data.domain.attributes
                              if attr in selected or
                              attr.varType == orange.VarTypes.String]  # ?? why strings
            if self.add_scores_to_output:
                scores = [self.scores[attr] for attr in selected_attrs]
                attrs = [copy_descriptor(attr) for attr in selected_attrs]
                for attr, score in zip(attrs, scores):
                    attr.attributes[method_name] = str(score)
                selected_attrs = attrs

            newdomain = orange.Domain(selected_attrs, self.data.domain.classVar)
            newdomain.addmetas(self.data.domain.getmetas())
            newdata = orange.ExampleTable(
                newdomain, self.data, name=self.data.name
            )
            self.send("Example table with selected genes",
                      newdata if selected_attrs else None)

            remaining_attrs = [attr for attr in self.data.domain.attributes
                               if attr in remaining]
            if self.add_scores_to_output:
                scores = [self.scores[attr] for attr in remaining_attrs]
                attrs = [copy_descriptor(attr) for attr in remaining_attrs]
                for attr, score in zip(attrs, scores):
                    attr.attributes[method_name] = str(scores)
                remaining_attrs = attrs

            newdomain = orange.Domain(remaining_attrs, self.data.domain.classVar)
            newdomain.addmetas(self.data.domain.getmetas())
            newdata = orange.ExampleTable(
                newdomain, self.data, name=self.data.name
            )
            self.send("Example table with remaining genes",
                      newdata if remaining_attrs else None)

            domain = orange.Domain([orange.StringVariable("label"),
                                    orange.FloatVariable(self.score_methods[self.method_index][0])],
                                    False)
            if selected_attrs:
                selected_genes = orange.ExampleTable(domain,
                            [[attr.name, self.scores.get(attr, 0)] for attr in selected_attrs])
            else:
                selected_genes = None
            self.send("Selected genes",  selected_genes)
            
        else:
            self.send("Example table with selected genes", None)
            self.send("Example table with remaining genes", None)
            self.send("Selected genes", None)
        self.data_changed_flag = False

    def on_target_changed(self):
        label, values = self.label_selection_widget.current_selection()

        if values is None:
            values = []

        if self.genes_in_columns:
            targets = [(label, t) for t in values]
        else:
            targets = values

        self.targets = targets
        self.current_target_selection = label, values
        # Save target label selection
        labels = [l for l, _ in self.data_labels]
        if label in labels:
            label_index = labels.index(label)
            self.target_selections[label_index] = values
        self.set_targets(targets)

    def on_label_activated(self, index):
        selection = self.target_selections[index]
        if not selection:
            selection = self.data_labels[index][1][:1]
        self.label_selection_widget.set_selection(index, selection)

    def on_genes_in_columns_change(self):
        self.closeContext("TargetSelection")
        self.update_targets_widget()
        items = [(label, v) for label, values in self.data_labels \
                                for v in values]
        if self.data_labels:
            label, values = self.data_labels[0] 
            self.current_target_selection = label, values[:1]
            self.target_selections = [values[1:] for _, values in self.data_labels]
            self.openContext("TargetSelection", set(items))
            self.label_selection_widget.set_selection(*self.current_target_selection)
        
    def on_scoring_method_changed(self):
        two_sample = self.score_methods[self.method_index][3]
        self.label_selection_widget.values_view.setDisabled(not two_sample)
        
    def settingsFromWidgetCallbackTargetSelection(self, handler, context):
        context.target_selections = self.target_selections
        context.current_target_selection = self.current_target_selection
        
    def settingsToWidgetCallbackTargetSelection(self, handler, context):
        self.target_selections = getattr(context, "target_selections", self.target_selections)
        self.current_target_selection = getattr(context, "current_target_selection", self.current_target_selection)


def copy_descriptor(desc):
    d = desc.clone()
    d.attributes = dict(desc.attributes)
    c = Orange.core.ClassifierFromVar()
    c.whichVar = desc
    d.get_value_from = c
    return d


if __name__=="__main__":
    import sys
    app = QApplication(sys.argv)
    data = orange.ExampleTable(os.path.expanduser("~/Documents/GDS636"))
    w = OWFeatureSelection()
    w.show()
    w.set_data(data)
    app.exec_()
    w.saveSettings()



import Orange

gds = Orange.bio.geo.GDS("GDS1676")
data = gds.getdata(sample_type="infection")
print "Genes: %d, Samples: %d" % (len(data), len(data.domain.attributes))

for a in data.domain.attributes:
    print a.name, a.attributes



"""
<name>Databases</name>
<description>Update of systems biology data and knowledge bases.</description>
<contact>Ales Erjavec</contact>
<priority>10</priority>
<icon>icons/Databases.svg</icon>
"""

from Orange.OrangeWidgets.OWDatabasesUpdate import *

NAME = "Databases"
DESCRIPTION = "Updates local system biology databases."
ICON = "icons/Databases.svg"
PRIORITY = 10

INPUTS = []
OUTPUTS = []

REPLACES = ["_bioinformatics.widgets.OWUpdateGenomicsDatabases.OWUpdateGenomicsDatabases"]


class OWUpdateGenomicsDatabases(OWDatabasesUpdate): 
    def __init__(self, parent=None, signalManager=None, name="Databases", **kwds):
        OWDatabasesUpdate.__init__(self, parent, signalManager, name, domains = \
                ["GO", "MeSH", "Taxonomy", "NCBI_geneinfo", "GEO", 
                 "dictybase", "OMIM", "HomoloGene", "Affy", "miRNA", "gene_sets",
                 "PPI"], **kwds)

if __name__ == "__main__":
    app = QApplication(sys.argv)
    w = OWUpdateGenomicsDatabases()
##    app.setMainWidget(w)
    w.show()
    app.exec_()
    w.saveSettings()



"""
==============================
OBO Ontology (:mod:`ontology`)
==============================

This module provides an interface for parsing, creating and manipulating of
`OBO ontologies <http://www.obofoundry.org/>`_.

Construct an ontology from scratch with custom terms ::

    >>> term = OBOObject("Term", id="foo:bar", name="Foo bar")
    >>> print term
    [Term]
    id: foo:bar
    name: Foo bar

    >>> ontology = OBOOntology()
    >>> ontology.add_object(term)
    >>> ontology.add_header_tag("created-by", "ales") # add a header tag
    >>> from StringIO import StringIO
    >>> buffer = StringIO()
    >>> ontology.write(buffer) # Save the ontology to a file like object
    >>> print buffer.getvalue() # Print the contents of the buffer
    created-by: ales
    <BLANKLINE>
    [Term]
    id: foo:bar
    name: Foo bar
    <BLANKLINE>

To load an ontology from a file, pass the file or filename to the
:class:`OBOOntology` constructor or call its load method ::

    >>> buffer.seek(0) # rewind
    >>> ontology = OBOOntology(buffer)
    >>> # Or equivalently
    >>> buffer.seek(0) # rewind
    >>> ontology = OBOOntology()
    >>> ontology.load(buffer)


See the definition of the `.obo file format <http://www.geneontology.org/GO.format.obo-1_2.shtml>`_.

"""

import re
import urllib2
import warnings
import keyword
from collections import defaultdict
from StringIO import StringIO


#: These are builtin OBO objects present in any ontology by default.
BUILTIN_OBO_OBJECTS = [
"""[Typedef]
id: is_a
name: is_a
range: OBO:TERM_OR_TYPE
domain: OBO:TERM_OR_TYPE
definition: The basic subclassing relationship [OBO:defs]""",

"""[Typedef]
id: disjoint_from
name: disjoint_from
range: OBO:TERM
domain: OBO:TERM
definition: Indicates that two classes are disjoint [OBO:defs]""",

"""[Typedef]
id: instance_of
name: instance_of
range: OBO:TERM
domain: OBO:INSTANCE
definition: Indicates the type of an instance [OBO:defs]""",

"""[Typedef]
id: inverse_of
name: inverse_of
range: OBO:TYPE
domain: OBO:TYPE
definition: Indicates that one relationship type is the inverse of another [OBO:defs]""",

"""[Typedef]
id: union_of
name: union_of
range: OBO:TERM
domain: OBO:TERM
definition: Indicates that a term is the union of several others [OBO:defs]""",

"""[Typedef]
id: intersection_of
name: intersection_of
range: OBO:TERM
domain: OBO:TERM
definition: Indicates that a term is the intersection of several others [OBO:defs]"""
]


def _split_and_strip(string, sep):
    """
    Split the `string` by separator `sep` in to two parts and strip
    any whitespace between the inner parts.

    """
    head, tail = _split_esc(string, sep)
    return head.rstrip(" "), tail.lstrip(" ")


def _rsplit_and_strip(string, sep):
    """
    Right split the `string` by separator `sep` in to two parts and
    strip any whitespace between the inner parts.

    """
    head, tail = _rsplit_esc(string, sep)
    return head.rstrip(" "), tail.lstrip(" ")


def _find_esc(string, char):
    i = string.find(char)
    while i != -1:
        if (i > 0 and string[i - 1] != "\\") or string[i - 1] != "\\":
            return i
        else:
            i = string.find(char, i + 1)
    return i


def _rfind_esc(string, char):
    i = string.rfind(char)
    while i != -1:
        if (i > 0 and string[i - 1] != "\\") or string[i - 1] != "\\":
            return i
        else:
            i = string.rfind(char, 0, i - 1)
    return i


def _split_esc(string, sep, _find_esc=_find_esc):
    i = _find_esc(string, sep)
    if i != -1:
        return string[:i], string[i + 1:]
    else:
        return string, ""


def _rsplit_esc(string, sep):
    i = _rfind_esc(string, sep)
    if i != -1:
        return string[:i], string[i + 1:]
    else:
        return string, ""


def parse_tag_value(tag_value_string):
    """
    Parse a tag value string and return a four-tuple containing
    a (tag, value, modifiers, comment). If comment or modifiers are
    not present the corresponding entry will be ``None``.

    >>> parse_tag_value("foo: bar {modifier=frob} ! Comment")
    ('foo', 'bar', 'modifier=frob', 'Comment')
    >>> parse_tag_value("foo: bar")
    ('foo', 'bar', None, None)
    >>> parse_tag_value("foo: bar [baz:0] { fizz=buzz } ! Comment")
    ('foo', 'bar [baz:0]', 'fizz=buzz', 'Comment')

    """
    comment = modifiers = None
    # First get rid of the comment if present
    if _rfind_esc(tag_value_string, "!") != -1:
        tag_value_string, comment = _rsplit_and_strip(tag_value_string, "!")

    # Split on the first unescaped ":"
    tag, value = _split_and_strip(tag_value_string, ":")

    # Split the value on { to get the modifiers if present
    value = value.rstrip()
    if value.endswith("}") and not value.endswith(r"\}") and \
            _rfind_esc(value, "{") != -1:
        value, modifiers = _rsplit_and_strip(value, "{")
        # remove closing } and any whitespace
        modifiers = modifiers[: -1].rstrip()

    return tag, value, modifiers, comment


class OBOObject(object):
    """
    A generic OBO object (e.g. Term, Typedef, Instance, ...).
    Example::

        >>> term = OBOObject(stanza_type="Term", id="FOO:001", name="bar")

        >>> term = OBOObject(
        ...     stanza_type="Term",
        ...     id="FOO:001",
        ...     name="bar",
        ...     def_="Example definition { modifier=frob } ! Comment"
        ... )
        ...

    An alternative way to specify tags in the constructor::

        >>> term = OBOObject(stanza_type="Term", id="FOO:001", name="bar",
        ...                  def_=("Example definition",
        ...                        [("modifier", "frob")],
        ...                        "Comment"))
        ...

    .. note::
        Note the use of ``def_`` to define the 'def' tag. This is to
        avoid the name clash with the python's ``def`` keyword.

    .. seealso:: :class:`Term` :class:`Typedef` :class:`Instance`

    """
    def __init__(self, stanza_type="Term", **kwargs):
        """
        Initialize from keyword arguments.
        """
        self.stanza_type = stanza_type
        self.tag_values = []
        self.values = {}

        sorted_tags = sorted(
            kwargs.iteritems(),
            key=lambda key_val: chr(1) if key_val[0] == "id" else key_val[0]
        )

        for tag, value in sorted_tags:
            if isinstance(value, basestring):
                tag, value, modifiers, comment = \
                    parse_tag_value(name_demangle(tag) + ": " + value)
            elif isinstance(value, tuple):
                tag, value, modifiers, comment = \
                    ((name_demangle(tag),) + value + (None, None))[:4]
            self.add_tag(tag, value, modifiers, comment)

        self.related = set()

    @property
    def is_annonymous(self):
        """
        Is this object anonymous.
        """
        value = self.get_values("is_annonymous")
        return bool(value)

    @property
    def id(self):
        """
        The id of this object.
        """
        value = self.get_values("id")
        return value[0] if value else None

    @property
    def name(self):
        """
        Name of this object
        """
        value = self.get_values("name")
        return value[0] if value else None

    def name_mangle(self, tag):
        return name_mangle(tag)

    def name_demangle(self, tag):
        return name_demangle(tag)

    def add_tag(self, tag, value, modifiers=None, comment=None):
        """
        Add `tag`, `value` pair to the object with optional modifiers and
        comment.

        Example::

            >>> term = OBOObject("Term")
            >>> term.add_tag("id", "FOO:002", comment="This is an id")
            >>> print term
            [Term]
            id: FOO:002 ! This is an id

        """
        tag = intern(tag)  # a small speed and memory benefit
        self.tag_values.append((tag, value, modifiers, comment))
        self.values.setdefault(tag, []).append(value)

    def add_tags(self, tag_value_iter):
        for tag, value, modifiers, comment in tag_value_iter:
            self.tag_values.append((tag, value, modifiers, comment))
            self.values.setdefault(tag, []).append(value)

    def update(self, other):
        """
        Update the term with tag value pairs from `other`
        (:class:`OBOObject`). The tag value pairs are appended to the
        end except for the `id` tag.

        """
        for tag, value, modifiers, comment in other.tag_values:
            if tag != "id":
                self.add_tag(tag, value, modifiers, comment)

    def get_values(self, tag):
        try:
            return self.values[tag]
        except KeyError:
            return []

    def tag_count(self):
        """
        Return the number of tags in this object.
        """
        return len(self.tag_values)

    def tags(self):
        """
        Return an list of all (tag, value, modifiers, comment) tuples.
        """
        return list(self.tag_values)

    def _format_single_tag(self, index):
        """
        Return a formated string representing index-th tag pair value.

        Example::

            >>> term = OBOObject(
            ...     "Term", id="FOO:001", name="bar",
            ...      def_="Example definition {modifier=frob} ! Comment")
            ...
            >>> term._format_single_tag(0)
            'id: FOO:001'
            >>> term._format_single_tag(1)
            'def: Example definition { modifier=frob } ! Comment'

        ..
            Why by index, and not by tag? Multiple tags are allowed.

        """
        tag, value, modifiers, comment = self.tag_values[index]
        res = ["%s: %s" % (tag, value)]
        if modifiers:
            res.append("{ %s }" % modifiers)
        if comment:
            res.append("! " + comment)
        return " ".join(res)

    def format_stanza(self):
        """
        Return a string stanza representation of this object.
        """
        stanza = ["[%s]" % self.stanza_type]
        for i in range(self.tag_count()):
            stanza.append(self._format_single_tag(i))
        return "\n".join(stanza)

    @classmethod
    def parse_stanza(cls, stanza):
        r'''
        Parse and return an OBOObject instance from a stanza string.

        >>> term = OBOObject.parse_stanza("""\
        ... [Term]
        ... id: FOO:001
        ... name: bar
        ... """)
        >>> print term.id, term.name
        FOO:001 bar

        '''
        lines = stanza.splitlines()
        stanza_type = lines[0].strip("[]")

        tag_values = [parse_tag_value(line) for line in lines[1:]
                      if ":" in line]

        obo = OBOObject(stanza_type)
        obo.add_tags(tag_values)
        return obo

    def related_objects(self):
        """
        Return a list of tuple pairs where the first element is
        relationship (typedef id) and the second object id whom the
        relationship applies to.

        """
        result = [(type_id, id)
                  for type_id in ["is_a"]  # TODO add other defined Typedef ids
                  for id in self.values.get(type_id, [])]

        result = result + [tuple(r.split(None, 1))
                           for r in self.values.get("relationship", [])]
        return result

    def __str__(self):
        """
        Return a string representation of the object in OBO format
        """
        return self.format_stanza()

    def __repr__(self):
        return ("{0.__name__}(id={1.id!r}, name={1.name}, ...)"
                .format(type(self), self))

    def __iter__(self):
        """
        Iterates over sub terms
        """
        return iter(self.related_objects())


class Term(OBOObject):
    """
    A 'Term' object in the ontology.
    """
    def __init__(self, *args, **kwargs):
        OBOObject.__init__(self, "Term", *args, **kwargs)


class Typedef(OBOObject):
    """
    A 'Typedef' object in the ontology.
    """
    def __init__(self, *args, **kwargs):
        OBOObject.__init__(self, "Typedef", *args, **kwargs)


class Instance(OBOObject):
    """
    An 'Instance' object in the ontology
    """
    def __init__(self, *args, **kwargs):
        OBOObject.__init__(self, "Instance", *args, **kwargs)


class OBOParser(object):
    r''' A simple parser for .obo files (inspired by xml.dom.pulldom)

    >>> from StringIO import StringIO
    >>> file = StringIO("""\
    ... header_tag: header_value
    ... [Term]
    ... id: FOO:001 { modifier=bar } ! comment
    ... """)
    >>> parser = OBOParser(file)
    >>> for event, value in parser:
    ...     print event, value
    ...
    HEADER_TAG ['header_tag', 'header_value']
    START_STANZA Term
    TAG_VALUE ('id', 'FOO:001', 'modifier=bar', 'comment')
    CLOSE_STANZA None

    '''
    def __init__(self, file):
        self.file = file

    def parse(self, progress_callback=None):
        """
        Parse the file and yield parse events.

        .. todo List events and values

        """
        data = self.file.read()
        header = data[: data.index("\n[")]
        body = data[data.index("\n[") + 1:]
        for line in header.splitlines():
            if line.strip():
                yield "HEADER_TAG", line.split(": ", 1)

        current = None
        #  For speed make these functions local
        startswith = str.startswith
        endswith = str.endswith
        parse_tag_value_ = parse_tag_value

        for line in body.splitlines():
            if startswith(line, "[") and endswith(line, "]"):
                yield "START_STANZA", line.strip("[]")
                current = line
            elif startswith(line, "!"):
                yield "COMMENT", line[1:]
            elif line:
                yield "TAG_VALUE", parse_tag_value_(line)
            else:  # empty line is the end of a term
                yield "CLOSE_STANZA", None
                current = None
        if current is not None:
            yield "CLOSE_STANZA", None

    def __iter__(self):
        """
        Iterate over parse events (same as parse())
        """
        return self.parse()


class OBOOntology(object):
    """
    An class representing an OBO ontology.

    :param file-like file:
        A optional file like object describing the ontology in obo format.

    """

    BUILTINS = BUILTIN_OBO_OBJECTS

    def __init__(self, file=None):
        self.objects = []
        self.header_tags = []
        self.id2term = {}
        self.alt2id = {}
        self._resolved_imports = []
        self._invalid_cache_flag = False
        self._related_to = {}

        # First load the built in OBO objects
        builtins = StringIO("\n" + "\n\n".join(self.BUILTINS) + "\n")
        self.load(builtins)
        if file:
            self.load(file)

    def add_object(self, obj):
        """
        Add an :class:`OBOObject` instance to this ontology.
        """
        if obj.id in self.id2term:
            raise ValueError("OBOObject with id: %s already in "
                             "the ontology" % obj.id)
        self.objects.append(obj)
        self.id2term[obj.id] = obj
        self._invalid_cache_flag = True

    def add_header_tag(self, tag, value):
        """
        Add header tag, value pair to this ontology.
        """
        self.header_tags.append((tag, value))

    def load(self, file, progress_callback=None):
        """
        Load terms from a file.

        :param file-like file:
            A file-like like object describing the ontology in obo format.
        :param function progress_callback:
            An optional function callback to report on the progress.

        """
        if isinstance(file, basestring):
            file = open(file, "rb")

        parser = OBOParser(file)
        current = None
        tag_values = []
        for event, value in parser.parse(progress_callback=progress_callback):
            if event == "TAG_VALUE":
                tag_values.append(value)
            elif event == "START_STANZA":
                current = OBOObject(value)
            elif event == "CLOSE_STANZA":
                current.add_tags(tag_values)
                self.add_object(current)
                current = None
                tag_values = []
            elif event == "HEADER_TAG":
                self.add_header_tag(*value)
            elif event != "COMMENT":
                raise Exception("Parse Error! Unknown parse "
                                "event {0}".format(event))

        imports = [value for tag, value in self.header_tags
                   if tag == "import"]

        if imports:
            warnings.warn("Import header tags are not supported")

#        while imports:
#            url = imports.pop(0)
#            if uri not in self._resolved_imports:
#                imported = self.parse_file(open(url, "rb"))
#                ontology.update(imported)
#                self._resolved_imports.append(uri)

    def dump(self, file):
        # deprecated use write
        self.write(file)

    def write(self, stream):
        """
        Write the contents of the ontology to a `file` in .obo format.

        :param file-like file:
            A file like object.

        """
        if isinstance(stream, basestring):
            stream = open(stream, "wb")

        for key, value in self.header_tags:
            stream.write(key + ": " + value + "\n")

        # Skip the builtins
        for obj in self.objects[len(self.BUILTINS):]:
            stream.write("\n")
            stream.write(obj.format_stanza())
            stream.write("\n")

    def update(self, other):
        """
        Update this ontology with the terms from `other`.
        """
        for term in other:
            if term.id in self:
                if not term.is_annonymous:
                    self.term(term.id).update(term)
                else:  # Do nothing
                    pass
            else:
                self.add_object(term)
        self._invalid_cache_flag = True

    def _cache_validate(self, force=False):
        """
        Update the relations cache if `self._invalid_cache` flag is set.
        """
        if self._invalid_cache_flag or force:
            self._cache_relations()

    def _cache_relations(self):
        """
        Collect all relations from parent to a child and store it in
        ``self._related_to`` member.

        """
        related_to = defaultdict(list)
        for obj in self.objects:
            for rel_type, id in self.related_terms(obj):
                term = self.term(id)
                related_to[term].append((rel_type, obj))

        self._related_to = related_to
        self._invalid_cache_flag = False

    def term(self, id):
        """
        Return the :class:`OBOObject` associated with this id.

        :param str id:
            Term id string.

        """
        if isinstance(id, basestring):
            if id in self.id2term:
                return self.id2term[id]
            elif id in self.alt2id:
                return self.id2term[self.alt2id[id]]
            else:
                raise ValueError("Unknown term id: %r" % id)
        elif isinstance(id, OBOObject):
            return id

    def terms(self):
        """
        Return all :class:`Term` instances in the ontology.
        """
        return [obj for obj in self.objects if obj.stanza_type == "Term"]

    def term_by_name(self, name):
        """
        Return the term with name `name`.
        """
        terms = [t for t in self.terms() if t.name == name]
        if len(terms) != 1:
            raise ValueError("Unknown term name: %r" % name)
        return terms[0]

    def typedefs(self):
        """
        Return all :class:`Typedef` instances in the ontology.
        """
        return [obj for obj in self.objects if obj.stanza_type == "Typedef"]

    def instances(self):
        """
        Return all :class:`Instance` instances in the ontology.
        """
        return [obj for obj in self.objects if obj.stanza_type == "Instance"]

    def root_terms(self):
        """
        Return all root terms (terms without any parents).
        """
        return [term for term in self.terms() if not self.parent_terms(term)]

    def related_terms(self, term):
        """
        Return a list of (`rel_type`, `term_id`) tuples where `rel_type` is
        relationship type (e.g. 'is_a', 'has_part', ...) and `term_id` is
        the id of the term in the relationship.

        """
        term = self.term(term) if not isinstance(term, OBOObject) else term
        related = [(tag, value)
                   for tag in ["is_a"]  # TODO: add other typedef ids
                   for value in term.values.get(tag, [])]
        relationships = term.values.get("relationship", [])
        for rel in relationships:
            related.append(tuple(rel.split(None, 1)))
        return related

    def edge_types(self):
        """
        Return a list of all edge types in the ontology.
        """
        return [obj.id for obj in self.objects if obj.stanza_type == "Typedef"]

    def parent_edges(self, term):
        """
        Return a list of (rel_type, parent_term) tuples.
        """
        term = self.term(term)
        parents = []
        for rel_type, parent in self.related_terms(term):
            parents.append((rel_type, self.term(parent)))
        return parents

    def child_edges(self, term):
        """
        Return a list of (rel_type, source_term) tuples.
        """
        self._cache_validate()
        term = self.term(term)
        return self._related_to.get(term, [])

    def super_terms(self, term):
        """
        Return a set of all super terms of `term` up to the most general one.
        """
        terms = self.parent_terms(term)
        visited = set()
        queue = set(terms)
        while queue:
            term = queue.pop()
            visited.add(term)
            queue.update(self.parent_terms(term) - visited)
        return visited

    def sub_terms(self, term):
        """
        Return a set of all sub terms for `term`.
        """
        terms = self.child_terms(term)
        visited = set()
        queue = set(terms)
        while queue:
            term = queue.pop()
            visited.add(term)
            queue.update(self.child_terms(term) - visited)
        return visited

    def child_terms(self, term):
        """
        Return a set of all child terms for this `term`.
        """
        self._cache_validate()
        term = self.term(term)
        children = []
        for rel_type, term in self.child_edges(term):
            children.append(term)
        return set(children)

    def parent_terms(self, term):
        """
        Return a set of all parent terms for this `term`.
        """
        term = self.term(term)
        parents = []
        for rel_type, id in self.parent_edges(term):
            parents.append(self.term(id))
        return set(parents)

    def relations(self):
        """
        Return a list of all relations in the ontology.
        """
        relations = []
        for obj in self.objects:
            for type_id, id in  obj.related:
                target_term = self.term(id)
            relations.append((obj, type_id, target_term))
        return relations

    def __len__(self):
        """
        Return the number of all objects in the ontology.
        """
        return len(self.objects)

    def __iter__(self):
        """
        Return an iterator over all objects in the ontology.
        """
        return iter(self.objects)

    def __contains__(self, oboid):
        return oboid in self.id2term

    def __getitem__(self, oboid):
        """
        Get the object by it's id `oboid`
        """
        return self.id2term[oboid]

    def to_network(self, terms=None):
        """
        Return an Orange.network.Network instance constructed from
        this ontology.

        """
        edge_types = self.edge_types()
        terms = self.terms()
        from Orange.orng import orngNetwork
        import orange

        network = orngNetwork.Network(len(terms), True, len(edge_types))
        network.objects = dict([(term.id, i) for i, term in enumerate(terms)])

        edges = defaultdict(set)
        for term in self.terms():
            related = self.related_terms(term)
            for relType, relTerm in related:
                edges[(term.id, relTerm)].add(relType)

        edgeitems = edges.items()
        for (src, dst), eTypes in edgeitems:
            network[src, dst] = [1 if e in eTypes else 0 for e in edge_types]

        domain = orange.Domain([orange.StringVariable("id"),
                                orange.StringVariable("name"),
                                orange.StringVariable("def"),
                                ], False)

        items = orange.ExampleTable(domain)
        for term in terms:
            ex = orange.Example(domain, [term.id, term.name, term.values.get("def", [""])[0]])
            items.append(ex)

        relationships = set([", ".join(sorted(eTypes)) for (_, _), eTypes in edgeitems])
        domain = orange.Domain([orange.FloatVariable("u"),
                                orange.FloatVariable("v"),
                                orange.EnumVariable("relationship", values=list(edge_types))
                                ], False)

        id2index = dict([(term.id, i + 1) for i, term in enumerate(terms)])
        links = orange.ExampleTable(domain)
        for (src, dst), eTypes in edgeitems:
            ex = orange.Example(domain, [id2index[src], id2index[dst], eTypes.pop()])
            links.append(ex)

        network.items = items
        network.links = links
        network.optimization = None
        return network

    def to_networkx(self, terms=None):
        """
        Return a NetworkX graph of this ontology.
        """
        import networkx
        graph = networkx.Graph()

        edge_types = self.edge_types()

        edge_colors = {"is_a": "red"}

        if terms is None:
            terms = self.terms()
        else:
            terms = [self.term(term) for term in terms]
            super_terms = [self.super_terms(term) for term in terms]
            terms = reduce(set.union, super_terms, set(terms))

        for term in terms:
            graph.add_node(term.id, name=term.name)

        for term in terms:
            for rel_type, rel_term in self.related_terms(term):
                rel_term = self.term(rel_term)
                if rel_term in terms:
                    graph.add_edge(term.id, rel_term.id, label=rel_type,
                                   color=edge_colors.get(rel_type, "blue"))

        return graph

    def to_graphviz(self, terms=None):
        """
        Return an pygraphviz.AGraph representation of the ontology.
        If `terms` is not `None` it must be a list of terms in the ontology.
        The graph will in this case contain only the super graph of those
        terms.

        """
        import pygraphviz as pgv
        graph = pgv.AGraph(directed=True, name="ontology")

        edge_types = self.edge_types()

        edge_colors = {"is_a": "red"}

        if terms is None:
            terms = self.terms()
        else:
            terms = [self.term(term) for term in terms]
            super_terms = [self.super_terms(term) for term in terms]
            terms = reduce(set.union, super_terms, set(terms))

        for term in terms:
            graph.add_node(term.id, label=term.name)

        for root in self.root_terms():
            node = graph.get_node(root.id)
            node.attr["rank"] = "max"

        for term in terms:
            for rel_type, rel_term in self.related_terms(term):
                rel_term = self.term(rel_term)
                if rel_term in terms:
                    graph.add_edge(term.id, rel_term.id, label=rel_type,
                                   color=edge_colors.get(rel_type, "blue"))

        return graph


def name_mangle(tag):
    """
    Mangle tag name if it conflicts with python keyword.

    >>> term.name_mangle("def"), term.name_mangle("class")
    ('def_', 'class_')

    """
    if keyword.iskeyword(tag):
        return tag + "_"
    else:
        return tag


def name_demangle(tag):
    """
    Reverse of `name_mangle`.
    """
    if tag.endswith("_") and keyword.iskeyword(tag[:-1]):
        return tag[:-1]
    else:
        return tag


def load(file):
    """
    Load an ontology from a .obo file.
    """
    return OBOOntology(file)


def foundry_ontologies():
    """
    Return a list of ontologies available from the OBOFoundry `website
    <http://www.obofoundry.org/>`_. The list contains a tuples of the form
    `(title, url)` for instance
    ``('Biological process', 'http://purl.obolibrary.org/obo/go.obo')``

    """
    stream = urllib2.urlopen("http://www.obofoundry.org/")
    text = stream.read()
    pattern = r'<td class=".+?">\s*<a href=".+?">(.+?)</a>\s*</td>\s*<td class=".+?">.*?</td>\s*<td class=".+?">.*?</td>\s*?<td class=".+?">\s*<a href="(.+?obo)">.+?</a>'
    return re.findall(pattern, text)


if __name__ == "__main__":
    import doctest
    stanza = '''[Term]
id: FOO:001
name: bar
'''
    seinfeld = StringIO("""
[Typedef]
id: parent

[Typedef]
id: child
inverse_of: parent ! not actually used yet

[Term]
id: 001
name: George

[Term]
id: 002
name: Estelle
relationship: parent 001 ! George

[Term]
id: 003
name: Frank
relationship: parent 001 ! George

""")  # TODO: fill the ontology with all characters
    term = OBOObject.parse_stanza(stanza)

    seinfeld = OBOOntology(seinfeld)
    print seinfeld.child_edges("001")

    doctest.testmod(extraglobs={"stanza": stanza, "term": term},
                    optionflags=doctest.ELLIPSIS)



import Orange.bio.gsea
import Orange.bio.gene
import Orange.bio.geneset

data = Orange.data.Table("iris")

gen1 = Orange.bio.geneset.GeneSets(dict([
    ("sepal", ["sepal length", "sepal width"]), 
    ("petal", ["petal length", "petal width", "petal color"])
    ]))

res = Orange.bio.gsea.run(data, gene_sets=gen1, matcher=Orange.bio.gene.matcher([]), min_size=2)
print "%5s  %6s %6s %s" % ("LABEL", "NES", "P-VAL", "GENES")
for gs,resu in res.items():
    print "%5s  %6.3f %6.3f %s" % (gs.id, resu["nes"], resu["p"], str(resu["genes"]))



import Orange
from Orange.bio import dicty, geneset, gsea, gene, geo

gds = geo.GDS("GDS10")
data = gds.getdata(transpose=True) 

matcher = gene.matcher([gene.GMKEGG("Homo sapiens")])
genesets = geneset.collections((("KEGG",), "Homo sapiens"))

#the number of permutations (n) should be much higher
res = gsea.run(data, gene_sets=genesets, matcher=matcher, 
    min_part=0.05, permutation="phenotype", n=10, 
    phen_desc=data.domain["tissue"], gene_desc=True) 

print
print "GSEA results (descriptor: tissue)"
print "%-40s %6s %6s %6s %7s" % ("LABEL", "NES", "FDR", "SIZE", "MATCHED") 
for gs, resu in sorted(res.items(), key=lambda x: x[1]["fdr"])[:10]: 
    print "%-40s %6.3f %6.3f %6d %7d" % (gs.name[:30],
        resu["nes"], resu["fdr"], resu["size"], resu["matched_size"]) 



## Automatically adapted for numpy.oldnumeric Oct 04, 2007 by 

"""
<name>Impute Profiles</name>
<description>Imputation and non-parametric smoothing of expression profiles.</description>
<icon>icons/ImputeProfiles.png</icon>
<priority>1065</priority>
<contact>Peter Juvan (peter.juvan@fri.uni-lj.si)</contact>
<prototype>1</prototype>
"""

import math

import numpy.oldnumeric as Numeric, numpy.oldnumeric.ma as MA

from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWWidget import *

from .. import chipimpute, chipstat
from .OWDataFiles import DataFiles

class OWImputeProfiles(OWWidget):
    settingsList  = ['impute', 'imputeK', 'smooth', 'windowSize', 'commitOnChange']

    def __init__(self, parent=None, signalManager = None):
        self.callbackDeposit = [] # deposit for OWGUI callback functions
        OWWidget.__init__(self, parent, signalManager, 'Impute & Loess Profiles')

        self._data = None       # exampleTable
        self._dataMA = None     # input 2d masked array
        self._chipdata = None   # [(dirname0, [et0, et1, ...]), ...]
        self._chipdataMA = []   # [(dirname0, [m2d0, m2d1, ...]), ...]
        self.impute = 1
        self.imputeK = 20
        self.smooth = 1
        self.windowSize = 3
        self.commitOnChange = 1
               
        # Settings
        self.loadSettings()

        # GUI
        self.mainArea.setFixedWidth(0)
        ca=QFrame(self.controlArea)
        gl=QGridLayout(ca,4,1,5)
        # info
        box = QVGroupBox("Info", ca)
        gl.addWidget(box, 0,0)
        self.infoa = QLabel("No examples on input", box)
        self.infob = QLabel("", box)
        QLabel("", box)
        self.infoc = QLabel("No structured data on input", box)
        self.infod = QLabel("", box)
        # KNN impute
        self.boxImpute = QVGroupBox("Impute missing values", ca)
        gl.addWidget(self.boxImpute, 1,0)
        OWGUI.checkBox(self.boxImpute, self, "impute", "KNN impute", tooltip="Impute missing values from values of K nearest neighbours.", callback=self.change)
##        self.sliderK = OWGUI.hSlider(self.boxImpute, self, "imputeK", box=None, minValue=1, maxValue=7744, step=1, callback=self.imputeChange, labelFormat=" K = %i", ticks=0)
##        self.sliderK = OWGUI.qwtHSlider(self.boxImpute, self, "imputeK", box=None, label="K", labelWidth=12, minValue=1, maxValue=7744, step=0.02, precision=0, callback=self.imputeChange, logarithmic=1, ticks=0, maxWidth=200)
##        self.sliderK = OWGUI.qwtHSlider(self.boxImpute, self, "imputeK", box=None, label="K", labelWidth=12, minValue=1, maxValue=100, step=1, precision=0, callback=self.imputeKChange, logarithmic=0, ticks=0, maxWidth=200)
        self.sliderK = OWGUI.qwtHSlider(self.boxImpute, self, "imputeK", box=None, label="K", labelWidth=15, minValue=1, maxValue=999, step=1, precision=0, callback=self.imputeKChange, logarithmic=0, ticks=0, maxWidth=None)
        self.boxImpute.setDisabled(1)
        # loess
        self.boxLoess = QVGroupBox("Smoothing", ca)
        gl.addWidget(self.boxLoess, 2,0)
        OWGUI.checkBox(self.boxLoess, self, "smooth", "Loess smoothing", tooltip="Loess profiles, impute missing columns.", callback=self.change)
        lbl = QLabel("Window size (number of points)", self.boxLoess)
        lbl.setAlignment(Qt.AlignHCenter)
        self.sliderW = OWGUI.qwtHSlider(self.boxLoess, self, "windowSize", box=None, label="W", labelWidth=15, minValue=1, maxValue=999, step=1, precision=0, callback=self.smoothWChange, logarithmic=0, ticks=0, maxWidth=None)
        self.boxLoess.setDisabled(1)
        # output
        box = QVGroupBox("Output", ca)
        gl.addWidget(box, 3,0)
        OWGUI.checkBox(box, self, 'commitOnChange', 'Commit data on selection change')
        self.commitBtn = OWGUI.button(box, self, "Commit", callback=self.senddata, disabled=1)

        self.inputs = [("Examples", ExampleTable, self.data), ("Structured Data", DataFiles, self.chipdata)]
        self.outputs = [("Examples", ExampleTable), ("Structured Data", DataFiles)]

        # data dependent variables
        self.numRowsMissing = 0
        self.numRowsMissingChipData = 0
        self.resize(100,100)

    def change(self):
        if self.commitOnChange:
            self.senddata(0)

    def imputeKChange(self):
        if self.commitOnChange and self.impute:
            self.senddata(0)

    def smoothWChange(self):
        if self.commitOnChange and self.smooth:
            self.senddata(0)

    def setGuiCommonExpChip(self):
        """Sets GUI features that depend on both inputs, e.g. max. values of sliders, commit button.
        """
        # calculate maxK, maxW
        maxK = 1e9
        maxW = 1e9
        if self._dataMA != None:
            maxK = min(max(self._dataMA.shape[0]/50, 50), self._dataMA.shape[0], maxK)
            maxW = min(self._dataMA.shape[1], maxW)
        if self._chipdataMA:
            maxK = min(max(self._chipdataMA[0][1][0].shape[0]/50., 50), self._chipdataMA[0][1][0].shape[0], maxK)
            maxW = min(self._chipdataMA[0][1][0].shape[1], maxW)
        if maxK == 1e9: maxK = 1
        if maxW == 1e9: maxW = 1
        # enable sliders and commit button
        if self._dataMA != None or self._chipdataMA:
            # set up sliders (only if data present)
            self.sliderK.setRange(1, maxK, 1)
            if self.imputeK > maxK:
                self.sliderK.setValue(maxK)
                self.imputeK = maxK
            self.sliderW.setRange(1, maxW, 1)
            if self.windowSize > maxW:
                self.sliderW.setValue(maxW)
                self.windowSize = maxW
            self.boxImpute.setEnabled(1)
            self.boxLoess.setEnabled(1)
            self.commitBtn.setEnabled(1)
        else:
            self.boxImpute.setDisabled(1)
            self.boxLoess.setDisabled(1)
            self.commitBtn.setDisabled(1)

    def data(self, data):
        if data != None:
            self._data = data
##            self._dataMA = chipstat.orng2ma(data)
            self._dataMA = data.toNumpyMA("a")[0]
            # info text
            self.infoa.setText("Examples: %i profiles on %i points" % (self._dataMA.shape[0], self._dataMA.shape[1]))
            numTotalMissing = int(Numeric.multiply.reduce(self._dataMA.shape) - MA.count(self._dataMA))
            if numTotalMissing > 0:
                numValsByCol = MA.count(self._dataMA, 0)
                numEmptyCol = Numeric.add.reduce(Numeric.where(numValsByCol==0, 1, 0))
                colNonEmpty = Numeric.compress(numValsByCol!=0, Numeric.arange(self._dataMA.shape[1]))
                dataRemEmptyCol = self._dataMA.take(colNonEmpty, 1)
                self.numRowsMissing = Numeric.add.reduce(Numeric.where(MA.count(dataRemEmptyCol, 1) < dataRemEmptyCol.shape[1], 1, 0))
                s1 = ""
                s2 = ""
                if numEmptyCol > 0: s1 = "s"
                if self.numRowsMissing > 0: s2 = "s"
                self.infob.setText("missing %i values, %i column%s completely, %i row%s partially" % (numTotalMissing, numEmptyCol, s1, self.numRowsMissing, s2))
            else:
                self.infob.setText("")
        else:
            self._data = None
            self._dataMA = None
            self.infoa.setText("No examples on input")
            self.infob.setText("")
            self.numRowsMissing = 0
        self.setGuiCommonExpChip()
        if self.commitOnChange:
            self.senddata(1)


    def chipdata(self, data):
        """Input data: [(dirname0, [et0, et1, ...]), ...]
        """
        self.numRowsMissingChipData = 0
        self._chipdataMA = []
        if data != None:
            self._chipdata = data
            numValsAll = 0
            numValsNonMasked = 0
            numFiles = 0
            numExamplesList = []
            attribDict = {}
            numColMissing = 0
            for (name, etList) in data:
                numFiles += len(etList)
                self._chipdataMA.append((name,[]))
                for et in etList:
                    attribDict.update(dict(zip(map(lambda x: x.name, et.domain.attributes), et.domain.attributes)))
                    numExamplesList.append(len(et))
                    etm = et.toNumpyMA("a")[0]
                    colNonMissingInd = Numeric.compress(Numeric.not_equal(MA.count(etm, 0), 0), Numeric.arange(etm.shape[1])) # indices of columns that are not completely missing
                    numColMissing += etm.shape[1] - colNonMissingInd.shape[0]
                    self.numRowsMissingChipData += int(Numeric.add.reduce(Numeric.less(MA.count(etm.take(colNonMissingInd, 1), 1), etm.shape[1])))
                    numValsAll += int(Numeric.multiply.reduce(etm.shape))
                    numValsNonMasked += int(MA.count(etm))
                    self._chipdataMA[-1][1].append(etm)
            # info text
            self.infoc.setText("Structured Data: %i data files with %i profiles on %i points" % (numFiles, numExamplesList[0], len(attribDict)))
            numTotalMissing = numValsAll-numValsNonMasked
            if numTotalMissing > 0:
                print numTotalMissing, numColMissing, self.numRowsMissingChipData
                print type(numTotalMissing), type(numColMissing), type(self.numRowsMissingChipData)
                self.infod.setText("missing %i values, %i column%s completely, %i row%s partially" % (numTotalMissing, numColMissing, ["","s"][numColMissing!=1], self.numRowsMissingChipData, ["","s"][self.numRowsMissingChipData!=1]))
            else:
                self.infod.setText("")                
        else:
            self._chipdata = None
            self.infoc.setText("No structured data on input")
            self.infod.setText("")
        self.setGuiCommonExpChip()
        if self.commitOnChange:
            self.senddata(2)


    def senddata(self, outputSelector=0):
        """outputSelector = [0: examples + chip data | 1: examples | 2: chip data]
        """
        assert outputSelector in [0,1,2]
        # progress bar settings: 1: examples, 2: chip data
        steps = 0
        if outputSelector in [0,1]:
            if self.impute:
                steps += self.numRowsMissing
            if self.smooth and self._dataMA != None:
                steps += self._dataMA.shape[0]
        if outputSelector in [0,2]:
            if self.impute:
                steps += self.numRowsMissingChipData
            if self.smooth and self._chipdataMA:
                steps += len(self._chipdataMA[0][1][0]) * reduce(lambda x,y: x+len(y[1]), self._chipdataMA, 0) #self._chipdataMA.shape[2]
        if steps == 0: steps = 1
        pbStep = 100./steps
        self.progressBarInit()
        if outputSelector in [0,1]:
            self._sendexampledata(pbStep)
        if outputSelector in [0,2]:
            self._sendchipdata(pbStep)
        self.progressBarFinished()

    def _sendexampledata(self, pbStep):
        if self._dataMA != None:
            ma2d = self._dataMA
            if self.impute:
                ma2d = chipimpute.kNNimputeMA(ma2d, int(self.imputeK), callback=lambda: self.progressBarAdvance(pbStep))
            if self.smooth:
                ws = max(1.1, int(self.windowSize)) # fixes bug in statc.loess
                ma2d = chipimpute.loessMA(ma2d, ws, 1, callback=lambda: self.progressBarAdvance(pbStep))
            et = chipstat.ma2orng_keepClassMetas(ma2d, self._data)
            et.name = self._data.name
            self.send("Examples", et)
        else:
            self.send("Examples", None)


    def _sendchipdata(self, pbStep):
        if self._chipdataMA:
            chipdataNew = []    # [(dirname0, [et0, et1, ...]), ...]
            ws = max(1.1, int(self.windowSize)) # fixes bug in statc.loess
            for sIdx, (name, etmList) in enumerate(self._chipdataMA):
                chipdataNew.append((name,[]))
                for eIdx, etm in enumerate(etmList):
                    if self.impute:
                        etm = chipimpute.kNNimputeMA(etm, int(self.imputeK), callback=lambda: self.progressBarAdvance(pbStep))
                    if self.smooth:
                        etm = chipimpute.loessMA(etm, ws, 1, callback=lambda: self.progressBarAdvance(pbStep))
##                   chipdataNew[-1][1].append(orange.ExampleTable(et.domain, etm))
                    etNew = chipstat.ma2orng_keepClassMetas(etm, self._chipdata[sIdx][1][eIdx])
                    etNew.name = self._chipdata[sIdx][1][eIdx].name
                    chipdataNew[-1][1].append(etNew)
            self.send("Structured Data", chipdataNew)
        else:
            self.send("Structured Data", None)



if __name__=="__main__":
    a=QApplication(sys.argv)
    ow=OWImputeProfiles()
    a.setMainWidget(ow)
    ow.show()
##    ow.data(orange.ExampleTable("meanExpr_ann_pkaC.tab"))
##    ow.data(orange.ExampleTable(r"C:\Documents and Settings\peterjuv\My Documents\Orange\ANOVA\DictyChipData_BR_ACS_100_yakApufA\pufA\pufA1.1.xls.tab"))
##    ow.data(orange.ExampleTable(r"C:\Documents and Settings\peterjuv\My Documents\Orange\ANOVA\potato\I\I_30m_1_1042_teh1.tab"))
    ow.data(orange.ExampleTable(r"c:\Documents and Settings\peterjuv\My Documents\STEROLTALK\Data Hs\2007-08-29 Banjo\- data\untr maxAbsDiff.tab"))
    a.exec_loop()
    ow.saveSettings()

##    from . import OWDataFiles
##    from Orange.orng import orngSignalManager
##    signalManager = orngSignalManager.SignalManager(0)
##    a=QApplication(sys.argv)
##    ow=OWImputeProfiles(signalManager = signalManager)
##    a.setMainWidget(ow)
##    ow.show()
##    ds = OWDataFiles.OWDataFiles(signalManager = signalManager)
####    ds.loadData(r"C:\Documents and Settings\peterjuv\My Documents\2005-08 NIB Potato\potato.attrib")
##    ds.loadData(r"C:\Documents and Settings\peterjuv\My Documents\2005-08 NIB Potato\potato.sub100.avg")
##    signalManager.addWidget(ow)
##    signalManager.addWidget(ds)
##    signalManager.setFreeze(1)
##    signalManager.addLink(ds, ow, 'Structured Data', 'Structured Data', 1)
##    signalManager.setFreeze(0)
##    a.exec_loop()
##    ow.saveSettings()



from Orange.bio import go

ontology = go.Ontology()

# Print names and definitions of all terms with "apoptosis" in the name
apoptosis = [term for term in ontology.terms.values()
             if "apoptosis" in term.name.lower()]
for term in apoptosis:
    print term.name, term.id
    print term.def_

# Load annotations for yeast.
annotations = go.Annotations("sgd", ontology=ontology)

res = annotations.get_enriched_terms(["YGR270W", "YIL075C", "YDL007W"])

gene = annotations.alias_mapper["YIL075C"]
print gene, "(YIL075C) directly annotated to the following terms:"
for a in annotations.gene_annotations[gene]:
    print ontology[a.GO_ID].name, "with evidence code", a.Evidence_Code

# Get all genes annotated to the same terms as YIL075C
ids = set([a.GO_ID for a in annotations.gene_annotations[gene]])
for termid in ids:
    ants = annotations.get_all_annotations(termid)
    genes = set([a.DB_Object_Symbol for a in ants])
    print ", ".join(genes), "annotated to", termid, ontology[termid].name
